<?php

namespace App\Http\Controllers\Backend\Categories;

use App\Traits\Authorizable;
use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Services\UploadService;
use App\Traits\Response;
use App\Utils\MyConstant;
use Illuminate\Http\Request;
use Yajra\DataTables\DataTables;
use Auth;

class CategoryController extends Controller
{
    use Response;
    //  use Authorizable;

    private Category $model;
    private Request $request;
    private $isUpdate = false;

    public function __construct(Request $request)
    {
        $this->middleware('permission:category.index|category.create|category.edit|category.delete', ['only' => ['index', 'show']]);
        $this->middleware('permission:category.create', ['only' => ['create', 'store']]);
        $this->middleware('permission:category.edit', ['only' => ['edit', 'update']]);
        $this->middleware('permission:category.delete', ['only' => ['destroy']]);
        $this->request = $request;
    }


    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        if ($this->request->expectsJson()) {
            if ($this->request->filled('id')) {
                $categories = Category::withCount('childrens')->with("media")->where('category_id', $this->request->input('id'));
            } else {
                $categories = Category::withCount('childrens')->with("media")->parents();
            }

            return Datatables::of($categories)
                ->editColumn('media', function (Category $category) {
                    if ($media = optional($category->getFirstMedia(MyConstant::THUMB_COLLECTION))->getResponsiveImageUrls(MyConstant::THUMB)) {
                        return $media[0];
                    }
                    return "";
                })
                ->addColumn('url', function (Category $category) {
                    return route('category.index', ['id' => $category->id]);
                })
                ->editColumn('active', function (Category $category) {
                    return $category->status;
                })
                ->editColumn("updated_at", function (Category $category) {
                    return $category->updated_at_formated;
                })
                ->rawColumns(['active'])
                ->make(true);
        }



        $contextmenu = array();
        $contextmenu['show'] = ['name' => 'Info', 'icon' => 'fas fa-info-circle'];
        if (Auth::user()->can("category.edit")) {
            $contextmenu['edit'] = ['name' => 'Edit', 'icon' => 'edit'];
        }
        if (Auth::user()->can("category.delete")) {
            $contextmenu['delete'] = ['name' => 'Delete', 'icon' => 'delete'];
            $contextmenu['selectall'] = ['name' => 'Selected All', 'icon' => 'fas fa-check-circle'];
            $contextmenu['deselectall'] = ['name' => 'Deselected All', 'icon' => 'fas fa-check'];
            $contextmenu['deleteall'] = ['name' => 'Delete Selected', 'icon' => 'delete'];
        }
        $contextmenu['sep1'] = "---------";
        $contextmenu['reload'] = ['name' => 'Reload', 'icon' => 'fas fa-sync'];
        $contextmenu['quit'] = ['name' => 'Quit', 'icon' => 'quit'];
        return view('Backend.category.index', ['contextmenu' => $contextmenu]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $categories =  Category::parents()->doesntHave('videos')->pluck('name', 'id')->all();
        return view('Backend.category.create', compact('categories'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store()
    {


        $this->validate($this->request, [
            'name' => ['required', 'max:255', 'unique:App\Models\Category,name'],
            'category_id' => 'numeric|nullable',
            'ord' => 'required|numeric|nullable',
            'active' => 'boolean',
            'meta_title' => 'nullable|max:150|min:10',
            'meta_keywords' => 'nullable|max:80|min:8',
            'meta_description' => 'nullable|max:200|min:25',
            'thumb_url' => 'nullable|exclude_unless:thumb,null|exclude_unless:thumb_edit,null|active_url',
            'thumb' => 'image|nullable',
            'thumb_edit' => 'nullable'
        ]);


        $this->model = new Category;
        $this->model->name = $this->request->input('name');
        $this->model->category_id = $this->request->input('category_id');
        $this->model->ord = $this->request->input('ord');
        $this->model->active = $this->request->boolean('active');
        $this->model->description = $this->request->input('description');
        $this->model->save();

        if ($this->request->anyFilled('meta_title', 'meta_keywords', 'meta_description')) {
            $this->model->setMetaTags(
                [
                    'title' => $this->request->input('meta_title'),
                    'keywords' => $this->request->input('meta_keywords'),
                    'description' => $this->request->input('meta_description')
                ]
            );
        }
        // $this->addMetatags();
        // $this->uploadImages();

        if ($this->request->anyFilled('thumb_edit', 'thumb_url') || $this->request->hasFile('thumb')) {
            $ImageHandler = new UploadService($this->model);
            if ($this->request->filled('thumb_edit')) {
                $ImageHandler->setFrom(UploadService::FILE_FROM_BASE64); //Base64
                $ImageHandler->setBase64($this->request->input('thumb_edit')); //Base64
                $ImageHandler->setExtension("png");
            } else if ($this->request->hasFile('thumb')) {
                $ImageHandler->setFrom(UploadService::FILE_FROM_INPUT); //File
                $ImageHandler->setFile('thumb'); //Thumb
            } else if ($this->request->filled('thumb_url')) {
                $ImageHandler->setFrom(UploadService::FILE_FROM_URL); //URL 
                $ImageHandler->setUrl($this->request->input('thumb_url')); //URL
            }
            $ImageHandler->setTitle($this->model->name);
            $ImageHandler->setCollection(MyConstant::THUMB_COLLECTION);
            $ImageHandler->save();
        }
        return redirect()->route('category.index')->with("success", __('message.create', ['name' => 'Category']));
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $category = Category::with(['metatags', 'media'])->withCount('childrens')->findOrFail($id);
        return view('Backend.category.show', compact('category'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $category = Category::with(['metatags', 'media'])->findOrFail($id);
        $categories = Category::parents()->doesntHave('videos')->pluck('name', 'id')->all();
        $metatags = $category->metatags;

        return view('Backend.category.edit', compact('category', 'categories', 'metatags'));
    }

    /**
     * Update the specified resource in storage.
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update($id)
    {

        $this->validate($this->request, [
            'name' => ['required', 'max:255', 'unique:App\Models\Category,name,' . $id],
            'category_id' => 'numeric|nullable',
            'ord' => 'required|numeric|nullable',
            'active' => 'boolean',
            'meta_title' => 'nullable|max:150|min:10',
            'meta_keywords' => 'nullable|max:80|min:8',
            'meta_description' => 'nullable|max:200|min:25',
            'thumb_url' => 'nullable|exclude_unless:thumb,null|exclude_unless:thumb_edit,null|active_url',
            'thumb' => 'image|nullable',
            'thumb_edit' => 'nullable'
        ]);

        $this->model = Category::findOrFail($id);
        $this->model->name = $this->request->input('name');
        $this->model->category_id = $this->request->input('category_id');
        $this->model->ord = $this->request->input('ord');
        $this->model->active = $this->request->boolean('active');
        $this->model->description = $this->request->input('description');
        $this->model->save();

        if ($this->request->anyFilled('meta_title', 'meta_keywords', 'meta_description')) {
            $this->model->setMetaTags(
                ['metatagable_id' => $this->model->id],
                [
                    'title' => $this->request->input('meta_title'),
                    'keywords' => $this->request->input('meta_keywords'),
                    'description' => $this->request->input('meta_description')
                ]
            );
        }
        if ($this->request->anyFilled('thumb_edit', 'thumb_url') || $this->request->hasFile('thumb')) {
            $this->model->clearMediaCollection(MyConstant::THUMB_COLLECTION);
            $ImageHandler = new UploadService($this->model);
            if ($this->request->filled('thumb_edit')) {
                $ImageHandler->setFrom(UploadService::FILE_FROM_BASE64); //Base64
                $ImageHandler->setBase64($this->request->input('thumb_edit')); //Base64
                $ImageHandler->setExtension("png");
            } else if ($this->request->hasFile('thumb')) {
                $ImageHandler->setFrom(UploadService::FILE_FROM_INPUT); //File
                $ImageHandler->setFile('thumb'); //Thumb
            } else if ($this->request->filled('thumb_url')) {
                $ImageHandler->setFrom(UploadService::FILE_FROM_URL); //URL 
                $ImageHandler->setUrl($this->request->input('thumb_url'), MyConstant::ALLOW_URL_IMAGE_UPLOAD); //URL
            }
            $ImageHandler->setTitle($this->model->name);
            $ImageHandler->setCollection(MyConstant::THUMB_COLLECTION);
            $ImageHandler->save();
        }


        return back()->with("success", __('message.edit', ['name' => 'Category']));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $category = Category::findOrFail($id);
        $category->delete();
        if ($this->request->wantsJson()) {
            return $this->success(['message' => __('message.delete', ['name' => 'Category'])]);
        }
        return redirect()->route('category.index')->with('success', __('message.delete', ['name' => 'Category']));
    }


    private function addMetaTags()
    {
        if ($this->request->anyFilled('meta_title', 'meta_keywords', 'meta_description')) {
            $this->model->metatags()->updateOrCreate(
                ['metatagable_id' => $this->model->id],
                [
                    'title' => $this->request->input('meta_title'),
                    'keywords' => $this->request->input('meta_keywords'),
                    'description' => $this->request->input('meta_description')
                ]
            );
        }
    }
}
