<?php

namespace App\Http\Controllers\Backend\Pages;

use App\Http\Controllers\Controller;
use App\Models\Page;
use App\Traits\Response;
use Illuminate\Http\Request;
use Yajra\DataTables\DataTables;

class PageController extends Controller
{
    use Response;

    public function __construct()
    {
        $this->middleware('permission:pages.index|pages.create|pages.edit|pages.delete', ['only' => ['index', 'show']]);
        $this->middleware('permission:pages.create', ['only' => ['create', 'store']]);
        $this->middleware('permission:pages.edit', ['only' => ['edit', 'update']]);
        $this->middleware('permission:pages.delete', ['only' => ['destroy']]);
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if ($request->wantsJson()) {
            $pages = Page::select(['id', 'title', 'active', 'ord', 'created_at', 'updated_at']);
            return DataTables::of($pages)
                ->editColumn("updated_at", function (Page $page) {
                    return $page->updated_at_formated;
                })
                ->editColumn('active', function (Page $page) {
                    return $page->status;
                })
                ->rawColumns(['active'])
                ->make(true);
        }

        $contextmenu = array();
        $contextmenu['show'] = ['name' => 'Info', 'icon' => 'fas fa-info-circle'];
        if (auth()->user()->can("pages.edit")) {
            $contextmenu['edit'] = ['name' => 'Edit', 'icon' => 'edit'];
        }
        if (auth()->user()->can("pages.delete")) {
            $contextmenu['delete'] = ['name' => 'Delete', 'icon' => 'delete'];
            $contextmenu['selectall'] = ['name' => 'Selected All', 'icon' => 'fas fa-check-circle'];
            $contextmenu['deselectall'] = ['name' => 'Deselected All', 'icon' => 'fas fa-check'];
            $contextmenu['deleteall'] = ['name' => 'Delete Selected', 'icon' => 'delete'];
        }
        $contextmenu['sep1'] = "---------";
        $contextmenu['reload'] = ['name' => 'Reload', 'icon' => 'fas fa-sync'];
        $contextmenu['quit'] = ['name' => 'Quit', 'icon' => 'quit'];
        return view('Backend.pages.index', ['contextmenu' => $contextmenu]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('Backend.pages.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'title' => ['required', 'max:255', 'unique:App\Models\Page,title'],
            'ord' => 'numeric|required',
            'active' => 'boolean',
            'url' => 'nullable|url',
            'content' => "nullable|required_if:url,null|string"
        ]);

        $page = new Page;
        $page->title = $request->input('title');
        $page->url = $request->input('url');
        $page->ord = $request->input('ord');
        $page->content = $request->filled('url') ? null : $request->input('content');
        $page->active = $request->boolean('active');
        $page->save();
        return redirect()->route('pages.index')->with("success", __('message.added', ['name' => 'Page']));
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Page  $page
     * @return \Illuminate\Http\Response
     */
    public function show(Page $page)
    {
        return view("Backend.pages.show", compact('page'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Page  $page
     * @return \Illuminate\Http\Response
     */
    public function edit(Page $page)
    {
        return view("Backend.pages.edit", compact('page'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Page  $page
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Page $page)
    {

        $this->validate($request, [
            'title' => ['required', 'max:255', 'unique:App\Models\Page,title,' . $page->id],
            'ord' => 'numeric|nullable',
            'active' => 'boolean',
            'url' => 'nullable|url',
            'content' => "nullable|required_if:url,null|string"
        ]);

        $page->title = $request->input('title');
        $page->url = $request->input('url');
        $page->ord = $request->input('ord');
        $page->content = $request->filled('url') ? null : $request->input('content');
        $page->active = $request->boolean('active');
        $page->save();
        return redirect()->route('pages.index')->with("success", __('message.edit', ['name' => 'Page']));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Page  $page
     * @return \Illuminate\Http\Response
     */
    public function destroy(Page $page)
    {
        $page->delete();
        if (request()->wantsJson()) {
            return $this->success(['message' => __('message.delete', ['name' => 'Page'])]);
        }
        return redirect()->route('pages.index')->with('success', __('message.delete', ['name' => 'Page']));
    }
}
