<?php

namespace App\Http\Controllers\Backend\Playlist;

use App\Http\Controllers\Controller;
use App\Models\Playlist;
use App\Models\Video;
use App\Traits\Response;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
use Yajra\DataTables\Facades\DataTables;
use Illuminate\Support\Facades\Auth;
use MyConstant;
use App\Services\UploadService;

class PlaylistController extends Controller
{
    use Response;

    private Request $request;

    private Model $model;
    private bool $isUpdate = false;


    public function __construct(Request $request)
    {
        $this->middleware('permission:playlists.index|playlists.create|playlists.edit|playlists.delete', ['only' => ['index', 'show']]);
        $this->middleware('permission:playlists.create', ['only' => ['create', 'store']]);
        $this->middleware('permission:playlists.edit', ['only' => ['edit', 'update']]);
        $this->middleware('permission:playlists.delete', ['only' => ['destroy']]);
        $this->request = $request;
    }



    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {

        if ($this->request->wantsJson()) {
            $playlists = Playlist::withCount('videos')->with('media');
            return Datatables::of($playlists)
                ->editColumn('media', function (Playlist $playlist) {
                    if($media = optional($playlist->getFirstMedia(MyConstant::THUMB_COLLECTION))->getResponsiveImageUrls(MyConstant::THUMB)){
                        return $media[0];
                    }
                    return "";
                })
                ->editColumn('active', function (Playlist $playlist) {
                    return $playlist->status;
                })
                ->editColumn("updated_at", function (Playlist $playlist) {
                    return $playlist->updated_at_formated;
                })
                ->rawColumns(['active'])
                ->make(true);
        }

        $contextmenu = array();
        $contextmenu['show'] = ['name' => 'Info', 'icon' => 'fas fa-info-circle'];
        if (Auth::user()->can("playlists.edit")) {
            $contextmenu['edit'] = ['name' => 'Edit', 'icon' => 'edit'];
        }
        if (Auth::user()->can("playlists.delete")) {
            $contextmenu['delete'] = ['name' => 'Delete', 'icon' => 'delete'];
            $contextmenu['selectall'] = ['name' => 'Selected All', 'icon' => 'fas fa-check-circle'];
            $contextmenu['deselectall'] = ['name' => 'Deselected All', 'icon' => 'fas fa-check'];
            $contextmenu['deleteall'] = ['name' => 'Delete Selected', 'icon' => 'delete'];
        }
        $contextmenu['sep1'] = "---------";
        $contextmenu['reload'] = ['name' => 'Reload', 'icon' => 'fas fa-sync'];
        $contextmenu['quit'] = ['name' => 'Quit', 'icon' => 'quit'];

        return view('Backend.playlists.index', compact('contextmenu'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('Backend.playlists.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @return \Illuminate\Http\Response
     */
    public function store()
    {
        $this->validate($this->request, [
            'name' => 'string|max:255|min:3|unique:App\Models\Playlist',
            'description' => 'nullable',
            'meta_title' => 'nullable|max:150|min:10',
            'meta_keywords' => 'nullable|max:80|min:8',
            'meta_description' => 'nullable|max:200|min:25',
            'active' => 'boolean',
        ]);

        $this->model  = new Playlist;
        $this->model->name = $this->request->input('name');
        $this->model->description = $this->request->input('description');
        $this->model->active = $this->request->boolean('active');
        $this->model->user()->associate($this->request->user());
        $this->model->save();
        $this->AddOrUpdateMetags();
        $this->imageUploadService();
        return redirect()->route('playlists.index')->with('success', __('message.create', ['name' => 'Playlist']));
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Playlist  $playlist
     * @return \Illuminate\Http\Response
     */
    public function show(Playlist $playlist)
    {
        if ($this->request->wantsJson()) {
            $videos = $playlist->videos();
            $videos = $videos->select('id', 'title', $videos->qualifyPivotColumn('updated_at'));
            return DataTables::of($videos)
                ->editColumn('updated_at', function (Video $video) {
                    return $video->updated_at_formated;
                })
                ->make(true);
        }
        $contextmenu = array();
        if(Auth::user()->can("videos.index")){
            $contextmenu['detail'] = ['name' => 'Info', 'icon' => 'fas fa-info-circle'];
        }
        if (Auth::user()->can("playlists.edit")) {
            $contextmenu['selectall'] = ['name' => 'Selected All', 'icon' => 'fas fa-check-circle'];
            $contextmenu['deselectall'] = ['name' => 'Deselected All', 'icon' => 'fas fa-check'];
            $contextmenu['remove_video_playlist'] = ['name' => 'Remove Selected', 'icon' => 'fas fa-minus-circle'];
        }
        $contextmenu['sep1'] = "---------";
        $contextmenu['reload'] = ['name' => 'Reload', 'icon' => 'fas fa-sync'];
        $contextmenu['quit'] = ['name' => 'Quit', 'icon' => 'quit'];
        return view('Backend.playlists.show', compact('playlist', 'contextmenu'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Playlist  $playlist
     * @return \Illuminate\Http\Response
     */
    public function edit(Playlist $playlist)
    {
        return view('Backend.playlists.edit', compact('playlist'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \App\Models\Playlist  $playlist
     * @return \Illuminate\Http\Response
     */
    public function update(Playlist $playlist)
    {

        $this->validate($this->request, [
            'name' => 'string|max:255|min:3|unique:App\Models\Playlist,name,' . $playlist->id,
            'description' => 'nullable',
            'meta_title' => 'nullable|max:150|min:10',
            'meta_keywords' => 'nullable|max:80|min:8',
            'meta_description' => 'nullable|max:200|min:25',
            'active' => 'boolean',
        ]);

        $this->model = $playlist;
        $this->model->name = $this->request->input('name');
        $this->model->description = $this->request->input('description');
        $this->model->active = $this->request->boolean('active');
        $this->model->user()->associate($this->request->user());
        $this->model->save();
        $this->AddOrUpdateMetags();
        $this->isUpdate = true;
        $this->imageUploadService();
        return back()->with('success')->with('success', __('message.update', ['name' => 'Playlist']));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Playlist  $playlist
     * @return \Illuminate\Http\Response
     */
    public function destroy(Playlist $playlist)
    {
        $playlist->delete();
        if ($this->request->wantsJson()) {
            return $this->success(__('message.delete', ['name' => 'Playlist']));
        }

        return redirect()->route('playlists.index')->with('success', __('message.delete', ['name' => 'Playlist']));
    }

    private function AddOrUpdateMetags()
    {
        if ($this->request->anyFilled('meta_title', 'meta_keywords', 'meta_description')) {
            $this->model->setMetaTags(
                ['metatagable_id' => $this->model->getKey()],
                [
                    'title' => $this->request->input('meta_title'),
                    'keywords' => $this->request->input('meta_keywords'),
                    'description' => $this->request->input('meta_description')
                ]
            );
        }
    }

    private function imageUploadService()
    {
        if ($this->request->anyFilled('thumb_edit', 'thumb_url') || $this->request->hasFile('thumb')) {
            if ($this->isUpdate) {
                $this->model->clearMediaCollection(MyConstant::THUMB_COLLECTION);
            }
            $ImageHandler = new UploadService($this->model);
            if ($this->request->filled('thumb_edit')) {
                $ImageHandler->setFrom(UploadService::FILE_FROM_BASE64); //Base64
                $ImageHandler->setBase64($this->request->input('thumb_edit')); //Base64
                $ImageHandler->setExtension("png");
            } else if ($this->request->hasFile('thumb')) {
                $ImageHandler->setFrom(UploadService::FILE_FROM_INPUT); //File
                $ImageHandler->setFile('thumb'); //Thumb
            } else if ($this->request->filled('thumb_url')) {
                $ImageHandler->setFrom(UploadService::FILE_FROM_URL); //URL 
                $ImageHandler->setUrl($this->request->input('thumb_url'), MyConstant::ALLOW_URL_IMAGE_UPLOAD); //URL
            }
            $ImageHandler->setTitle($this->model->name);
            $ImageHandler->setCollection(MyConstant::THUMB_COLLECTION);
            $ImageHandler->save();
        }
    }
}
