<?php

namespace App\Http\Controllers\Backend\Users;

use App\Http\Controllers\Controller;
use App\Models\User;
use Illuminate\Http\Request;
use App\Traits\Response;
use Yajra\DataTables\DataTables;
use Spatie\Permission\Models\Role;
use Illuminate\Support\Facades\Auth;
use Spatie\Permission\Models\Permission;
use Illuminate\Validation\Rule;
use App\Utils\MyConstant;

class RoleController extends Controller
{
    use Response;

    public function __construct()
    {
        $this->middleware('permission:roles.index|roles.create|roles.edit|roles.delete', ['only' => ['index', 'show']]);
        $this->middleware('permission:roles.create', ['only' => ['create', 'store']]);
        $this->middleware('permission:roles.edit', ['only' => ['edit', 'update']]);
        $this->middleware('permission:roles.delete', ['only' => ['destroy']]);
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if ($request->wantsJson()) {
            $roles = Role::query();
            return DataTables::of($roles)
                ->make(true);
        }

        $contextmenu = array();
        $contextmenu['show'] = ['name' => 'Info', 'icon' => 'fas fa-info-circle'];
        if (Auth::user()->can("category.edit")) {
            $contextmenu['edit'] = ['name' => 'Edit', 'icon' => 'edit'];
        }
        if (Auth::user()->can("category.delete")) {
            $contextmenu['delete'] = ['name' => 'Delete', 'icon' => 'delete'];
            $contextmenu['selectall'] = ['name' => 'Selected All', 'icon' => 'fas fa-check-circle'];
            $contextmenu['deselectall'] = ['name' => 'Deselected All', 'icon' => 'fas fa-check'];
            $contextmenu['deleteall'] = ['name' => 'Delete Selected', 'icon' => 'delete'];
        }
        $contextmenu['sep1'] = "---------";
        $contextmenu['reload'] = ['name' => 'Reload', 'icon' => 'fas fa-sync'];
        $contextmenu['quit'] = ['name' => 'Quit', 'icon' => 'quit'];
        return view('Backend.roles.index', compact('contextmenu'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $permissions = User::getRolesAndPermission();
        return view("Backend.roles.create", compact('permissions'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'name' => 'required|unique:Spatie\Permission\Models\Role,name',
        ]);

        $role = Role::create([
            'name' => $request->input('name'),
            'permissions' => ['required', 'array', Rule::in(Permission::pluck('name')->all())]
        ]);


        if ($request->filled('permissions')) {
            $role->syncPermissions($request->input('permissions'));
        }
        return redirect()->route('roles.index')->with('success', __('message.create', ['name' => "Role"]));
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $userpermissions = [];
        $permissions = User::getRolesAndPermission();
        $role = Role::findOrFail($id);
        $userpermissions = $role->permissions()->pluck('name')->all();
        $constSuperAdmin = MyConstant::ROLE_SUPERADMIN;
        return view('Backend.roles.show', compact('role', 'permissions', 'userpermissions','constSuperAdmin'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $permissions = User::getRolesAndPermission();
        $role = Role::where('name', '!=', MyConstant::ROLE_SUPERADMIN)->findOrFail($id);
        $userpermissions = $role->permissions()->pluck('name')->all();
        return view('Backend.roles.edit', compact('role', 'permissions', 'userpermissions'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'name' => 'required|unique:Spatie\Permission\Models\Role,name,' . $id,
            'permissions' => ['nullable', 'array', Rule::in(Permission::pluck('name')->all())]
        ]);

        $role = Role::where('name', "!=", MyConstant::ROLE_SUPERADMIN)->findOrFail($id);
        $role->name = $request->input('name');
        $role->save();
        if ($request->filled('permissions')) {
            $role->syncPermissions($request->input('permissions'));
        }
        return redirect()->route('roles.index')->with('success', __('message.update', ['name' => 'Role']));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request, $id)
    {
        $role =  Role::where('name', "!=", MyConstant::ROLE_SUPERADMIN)->findOrFail($id);
        $role->delete();
        if ($request->wantsJson()) {
            return $this->success(['message' => __('message.delete', ['name' => 'Role'])]);
        }
        return redirect()->route('roles.index')->with('success', __('message.delete', ['name' => 'Role']));
    }
}
