<?php

namespace App\Http\Controllers\Backend\Users;

use App\Http\Controllers\Controller;
use App\Models\Admin;
use App\Models\User;
use App\Traits\Response;
use App\Utils\MyConstant;
use Arr;
use Auth;
use Hash;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use Spatie\Permission\Models\Role;
use Yajra\DataTables\DataTables;

class UserController extends Controller
{
    use Response;

    public function __construct()
    {
        $this->middleware('permission:users.index|users.create|users.edit|users.delete', ['only' => ['index', 'show']]);
        $this->middleware('permission:users.create', ['only' => ['create', 'store']]);
        $this->middleware('permission:users.edit', ['only' => ['edit', 'update']]);
        $this->middleware('permission:users.delete', ['only' => ['destroy']]);
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {

        if ($request->wantsJson()) {
            $admins = User::with('roles');
            return DataTables::of($admins)
                ->make(true);
        }

        $contextmenu = array();
        $contextmenu['show'] = ['name' => 'Info', 'icon' => 'fas fa-info-circle'];
        if (Auth::user()->can("users.edit")) {
            $contextmenu['edit'] = ['name' => 'Edit', 'icon' => 'edit'];
        }
        if (Auth::user()->can("users.delete")) {
            $contextmenu['delete'] = ['name' => 'Delete', 'icon' => 'delete'];
            $contextmenu['selectall'] = ['name' => 'Selected All', 'icon' => 'fas fa-check-circle'];
            $contextmenu['deselectall'] = ['name' => 'Deselected All', 'icon' => 'fas fa-check'];
            $contextmenu['deleteall'] = ['name' => 'Delete Selected', 'icon' => 'delete'];
            $contextmenu['sep1'] = "---------";
        }
        $contextmenu['reload'] = ['name' => 'Reload', 'icon' => 'fas fa-sync'];
        $contextmenu['quit'] = ['name' => 'Quit', 'icon' => 'quit'];

        $IsSuperAdmin = $this->isSuperAdmin() ? "Y" : "N";


        return view('Backend.users.index', compact('contextmenu', 'IsSuperAdmin'));
    }

    /**
     *  SHow 
     * 
     * @return 
     */

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $rolesAll = Role::when((!$this->isSuperAdmin()), function (Builder $query) {
            $query->whereNotIn('name', [MyConstant::ROLE_SUPERADMIN]);
        })->pluck('name', 'name')->all();

        return view('Backend.users.create', compact('rolesAll'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
        $this->validate($request, [
            'name' => 'required',
            'email' => 'required|email|unique:App\Models\User,email',
            'password' => 'required|confirmed',
            'roles' => ['required', 'array', Rule::in(Role::pluck('name')->all())]
        ]);

        $admin = User::create([
            'name' => $request->input('name'),
            'email' => $request->input('email'),
            'password' => Hash::make($request->input('password')),
        ]);
        $admin->assignRole($request->input('roles'));
        return redirect()->route('users.index')->with('success', 'User created successfully');
    }

    /**
     * Display the specified resource.
     *
     * @param  User $user
     * @return \Illuminate\Http\Response
     */
    public function show(User $user)
    {
        if (request()->wantsJson()) {
            $videos = $user->videos();
            $videos = $videos->select('id', 'title', 'updated_at');
            return DataTables::of($videos)
                ->make(true);
        }
        $contextmenu = array();
        if(Auth::user()->can("videos.index")){
            $contextmenu['detail'] = ['name' => 'Info', 'icon' => 'fas fa-info-circle'];
        }
        $contextmenu['sep1'] = "---------";
        $contextmenu['reload'] = ['name' => 'Reload', 'icon' => 'fas fa-sync'];
        $contextmenu['quit'] = ['name' => 'Quit', 'icon' => 'quit'];
        return view('Backend.Users.show', compact('user','contextmenu'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $admin =  User::when((!$this->isSuperAdmin()), function (Builder $query) {
            $query->whereDoesntHave('roles', function (Builder $query) {
                $query->where('name', MyConstant::ROLE_SUPERADMIN);
            });
        })->findOrFail($id);

        $rolesAll = Role::when((!$this->isSuperAdmin()), function (Builder $query) {
            $query->whereNotIn('name', [MyConstant::ROLE_SUPERADMIN]);
        })->pluck('name', 'name')->all();

        $userRole = $admin->roles->pluck('name', 'name')->all();
        return view('Backend.users.edit', compact('rolesAll', 'userRole', 'admin'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'name' => 'required',
            'email' => 'required|email|unique:App\Models\User,email,' . $id,
            'password' => 'nullable|confirmed',
            'roles' => ['required', 'array', Rule::in(Role::pluck('name')->all())]
        ]);
        $admin =  User::when((!$this->isSuperAdmin()), function (Builder $query) {
            $query->whereDoesntHave('roles', function (Builder $query) {
                $query->where('name', MyConstant::ROLE_SUPERADMIN);
            });
        })->findOrFail($id);

        $admin->name = $request->input('name');
        $admin->email = $request->input('email');
        if ($request->filled('password')) {
            $admin->password = Hash::make($request->input('password'));
        }
        $admin->save();
        if (Auth::user()->id != $admin->id) {
            $admin->syncRoles($request->input('roles'));
        }
        return redirect()->route('users.index')->with('success', __('message.Update Successfuly'));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request, $id)
    {

        if ($id == Auth::user()->id) {
            if ($request->wantsJson()) {
                return $this->failure([
                    'message' => __('message.you_cant', ['name' => 'delete own account'])
                ]);
            }
            return redirect()->route('users.index')->with('success', __('message.you_cant', ['name' => 'delete own account']));
        }

        try {

            User::when((!$this->isSuperAdmin()), function (Builder $query) {
                $query->whereDoesntHave('roles', function (Builder $query) {
                    $query->where('name', MyConstant::ROLE_SUPERADMIN);
                });
            })->findOrFail($id)->delete();
        } catch (ModelNotFoundException $e) {

            if ($request->wantsJson()) {
                return $this->failure([
                    'message' => __("message.delete_failed", ['name' => "User"])
                ]);
            }
            return redirect()->route('users.index')->with('success', _("message.delete_failed", ['name' => "User"]));
        }

        if ($request->wantsJson()) {
            return $this->success([
                'message' => __("message.delete", ['name' => "User"])
            ]);
        }

        return redirect()->route('users.index')->with('success', __('message.delete', ['name' => 'User']));
    }


    private function isSuperAdmin()
    {
        if (Auth::user()->hasRole(MyConstant::ROLE_SUPERADMIN)) {
            return true;
        }
        return false;
    }
}
