<?php

namespace App\Http\Controllers\Backend\Videos;

use App\Http\Controllers\Controller;
use App\Jobs\VideoUploadingJob;
use Illuminate\Http\Request;
use App\Models\Category;
use App\Traits\Response;
use Str;
use Illuminate\Support\Facades\Validator;
use Carbon\Carbon;
use DataTables;
use App\Models\Video;
use Storage;
use App\Services\UploadService;
use App\Utils\MyConstant;
use App\Utils\PendingVideo;
use Illuminate\Validation\Rule;
use Spatie\MediaLibrary\Support\File as FileHelper;
use Pion\Laravel\ChunkUpload\Exceptions\UploadMissingFileException;
use Pion\Laravel\ChunkUpload\Receiver\FileReceiver;
use Pion\Laravel\ChunkUpload\Handler\HandlerFactory;

class MultipleUploadController extends Controller
{
    use Response;

    protected Request $request;

    private Video $model;

    private string $disk = 'local';

    private ?string $dirPath = 'tmp';

    private ?string $replacePrefix = null;

    public function __construct(Request $request)
    {
        $this->middleware('permission:videos.create');
        $this->request = $request;
    }

    public function index()
    {

        if ($this->request->wantsJson()) {
            $this->disk = "local";
            $this->dirPath = "fm";
            $this->replacePrefix = $this->request->input('removeprefix');

            $videos = $this->getVideos();
            return DataTables::of($videos)
                ->addColumn('checkbox', "")
                ->addColumn('play', "")
                ->make(true);
        }

        $filesystem = collect(config('filesystems.disks'));
        $disks = $filesystem->mapWithKeys(function ($v, $k) {
            return [$k => Str::upper($k)];
        })->only("local");

        $categories = Category::doesntHave('childrens')->latest()->pluck('name', 'id');
        return view('Backend.videos.multiple', compact('categories', 'disks'));
    }

    public function fmDelete(Request $request)
    {

        if ($path = $request->input('path')) {
            $disk = Storage::disk("local");
            if ($disk->exists($path) && $disk->delete($path)) {
                return $this->success();
            }
        }
        return $this->failure();
    }

    public function upload()
    {

        $filesystem = collect(config('filesystems.disks'));
        $disks = $filesystem->keys()->all();

        $validator = Validator::make($this->request->all(), [
            'title' => ['required', 'max:255'],
            'category_id' => 'numeric|nullable',
            'disk' => ['required', Rule::in($disks)],
            'path' => 'required|string'
        ]);

        if ($validator->fails()) {
            return $this->failure(['message' => $validator->errors()->__toString()]);
        }

        $this->model = new Video;
        $this->model->title = $this->request->input('title');
        $this->model->is_download = $this->request->boolean('is_download');
        $this->model->featured = $this->request->boolean('featured');
        $this->model->active = false;
        $this->model->published_at = ($this->request->input('published_at')) ? Carbon::createFromFormat("Y-m-d H:i", $this->request->input('published_at')) : now();
        $this->model->is_uploaded = 2;
        $this->model->user()->associate($this->request->user());
        $this->model->category()->associate($this->request->input('category_id'));
        $this->model->save();


        $pendingVideo = new PendingVideo;
        $pendingVideo->video = $this->model;
        $pendingVideo->isActive = $this->request->boolean('active');
        $pendingVideo->videoPath = $this->request->input('path');
        $pendingVideo->isVideoUrl = false;

        if ($this->request->boolean('converter')) {
            $pendingVideo->conversion = true;
            VideoUploadingJob::dispatch($pendingVideo);
        } else {
            $pendingVideo->conversion = false;
            VideoUploadingJob::dispatchSync($pendingVideo);
        }

        return $this->success();
    }

    public function uploadLargeFiles(Request $request)
    {
        $receiver = new FileReceiver('file', $request, HandlerFactory::classFromRequest($request));
        if ($receiver->isUploaded() === false) {
            throw new UploadMissingFileException();
        }
        $fileReceived = $receiver->receive(); // receive file
        if ($fileReceived->isFinished()) {
            $file = $fileReceived->getFile();
            $dir = $request->input('dir') ?: "fm";
            $path = config('filesystems.disks.local.root') . '/' . $dir;
            $filename = $file->getClientOriginalName();
            $file->move($path, $filename);
            return $this->success(['path' => $dir . '/' . $filename]);
        }
        $handler = $fileReceived->handler();
        return response()->json([
            "done" => $handler->getPercentageDone(),
        ]);
    }


    private function getVideos()
    {
        $videos = Storage::disk($this->disk)->files($this->dirPath);
        $videos = collect($videos);

        $videos = $videos->reject(function ($v) {
            $fileExtension = Str::upper(pathinfo($v, PATHINFO_EXTENSION));
            $extension = ['MP4', 'WMV', 'AVI', 'MKV', 'WEBM'];
            return !in_array($fileExtension, $extension);
        });

        $videos = $videos->map(function ($v) {
            $pathPrefix = Storage::disk($this->disk)->path("");
            $path = Str::replaceFirst($pathPrefix, '', $v);
            return [
                'id' => Str::uuid()->toString(),
                'name' => Str::title(Str::remove(explode(',', $this->replacePrefix), pathinfo($v, PATHINFO_FILENAME))),
                'disk' => $this->disk,
                'size' => FileHelper::getHumanReadableSize(Storage::disk($this->disk)->size($path)),
                'modifiedAt' => Carbon::parse(Storage::disk($this->disk)->lastModified($path))->format('d M, y H:i'),
                'path' => $path,
                'url' => Storage::disk($this->disk)->url($path)
            ];
        });

        return $videos;
    }
}
