<?php

namespace App\Http\Controllers\Backend\Videos;

use App\Http\Controllers\Controller;
use App\Jobs\VideoUploadingJob;
use App\Models\Video;
use Illuminate\Http\Request;
use Yajra\DataTables\Facades\DataTables;
use App\Services\UploadService;
use App\Utils\MyConstant;
use Auth;
use Str;
use App\Models\Category;
use App\Traits\Response;
use App\Utils\PendingVideo;
use Carbon\Carbon;

class VidController extends Controller
{
    use Response;
    private Request $request;

    private Video $model;

    private bool $isUpdate = false;


    public function __construct(Request $request)
    {
        $this->middleware('permission:videos.index|videos.create|videos.edit|videos.delete', ['only' => ['index', 'show']]);
        $this->middleware('permission:videos.create', ['only' => ['create', 'store']]);
        $this->middleware('permission:videos.edit', ['only' => ['edit', 'update']]);
        $this->middleware('permission:videos.delete', ['only' => ['destroy']]);
        $this->request = $request;
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {

        if ($request->wantsJson()) {
            $videos = Video::with(['media', 'category:id,name', 'user:id,name']);
            return DataTables::of($videos)
                ->removeColumn(['description', 'tags', 'created_at'])
                ->editColumn('media', function (Video $video) {
                    if ($media = optional($video->getFirstMedia(MyConstant::THUMB_COLLECTION))->getResponsiveImageUrls(MyConstant::THUMB)) {
                        return $media[0];
                    }
                    return "";
                })
                ->editColumn("is_uploaded", function (Video $video) {
                    return $video->upload_status;
                })
                ->editColumn('active', function (Video $video) {
                    return $video->active_status;;
                })
                ->editColumn('published_at', function (Video $video) {
                    return $video->published_at_status;
                })
                ->editColumn('updated_at', function (Video $video) {
                    return $video->updated_at_formated;
                })
                ->editColumn('featured', function (Video $video) {
                    return $video->featured ? '<span class="badge badge-success">Active</span>' : '<span class="badge badge-warning">Deactive</span>';
                })
                ->editColumn('is_download', function (Video $video) {
                    return $video->is_download ? '<span class="badge badge-success">Enabled</span>' : '<span class="badge badge-warning">Disabled</span>';
                })
                ->rawColumns(['active', 'featured', 'is_uploaded', 'published_at', 'is_download'])
                ->make(true);
        }

        $contextmenu = array();
        $contextmenu['show'] = ['name' => 'Info', 'icon' => 'fas fa-info-circle'];

        if (Auth::user()->can("videos.edit")) {
            $contextmenu['edit'] = ['name' => 'Edit', 'icon' => 'edit'];
        }

        if (Auth::user()->canAny(['videos.delete', 'videos.edit'])) {
            if (Auth::user()->can("videos.delete")) {
                $contextmenu['delete'] = ['name' => 'Delete', 'icon' => 'delete'];
            }
            $contextmenu['selectall'] = ['name' => 'Selected All', 'icon' => 'fas fa-check-circle'];
            $contextmenu['deselectall'] = ['name' => 'Deselected All', 'icon' => 'fas fa-check'];
            if (Auth::user()->can("videos.delete")) {
                $contextmenu['deleteall'] = ['name' => 'Delete Selected', 'icon' => 'delete'];
            }
        }

        if (Auth::user()->can('playlists.edit')) {
            $contextmenu['add_video_playlist'] = ['name' => 'Add Video to Playlist', 'icon' => 'fas fa-list'];
        }

        $contextmenu['sep'] = "";
        $contextmenu['reload'] = ['name' => 'Reload', 'icon' => 'fas fa-sync'];
        $contextmenu['quit'] = ['name' => 'Quit', 'icon' => 'quit'];

        return view('Backend.videos.index', compact('contextmenu'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $categories = Category::doesntHave('childrens')->latest()->pluck('name', 'id');
        return view('Backend.videos.create', compact('categories'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store()
    {
        $this->validate($this->request, [
            'title' => ['required', 'max:255'],
            'category_id' => 'numeric|nullable',
            'active' => 'boolean',
            'converter' => 'boolean',
            'is_download' => 'boolean',
            'description' => 'nullable',
            'meta_title' => 'nullable|max:150|min:10',
            'meta_keywords' => 'nullable|max:80|min:8',
            'meta_description' => 'nullable|max:200|min:25',
            'file_url' => 'nullable|exclude_unless:file_path,null|active_url',
            'file_path' => 'nullable|required_if:file_url,null|string',
            'thumb_url' => 'nullable|exclude_unless:thumb,null|exclude_unless:thumb_edit,null|active_url',
            'thumb' => 'image|nullable',
            'thumb_edit' => 'nullable'
        ]);

        $this->model = new Video;
        $this->model->title = $this->request->input('title');
        $this->model->tags = $this->request->input('tags');
        $this->model->description = $this->request->input('description');
        $this->model->url = $this->request->boolean('copyfile') ? null : $this->request->input('file_url');
        $this->model->is_download = $this->request->boolean('is_download');
        $this->model->featured = $this->request->boolean('featured');
        $this->model->active = $this->request->boolean('copyfile') ? false :  $this->request->boolean("active");
        $this->model->user()->associate($this->request->user());
        $this->model->category()->associate($this->request->input('category_id'));
        $this->model->is_uploaded = $this->request->boolean('copyfile') ? 2 : 1;
        $this->model->published_at = ($this->request->input('published_at')) ? Carbon::createFromFormat("Y-m-d H:i", $this->request->input('published_at')) : now();
        $this->model->save();
        $this->uploadService();
        return redirect()->route('videos.create')->with('success', __('message.added', ['name' => 'Last File: ' . $this->request->input('title') . ' - ']));
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Video  $video
     * @return \Illuminate\Http\Response
     */
    public function show(Video $video)
    {
        $metatags = $video->metatags;
        $categories = Category::doesntHave('childrens')->latest()->pluck('name', 'id');
        $poster = optional($video->getFirstMedia(MyConstant::THUMB_COLLECTION))->getUrl(MyConstant::THUMB);
        if (!$poster) {
            $poster = asset('images/default.jpg');
        }
        //Player Config
        $playerConfig = [
            'title' => $video->title,
        ];

        return view("Backend.videos.show", compact('video', 'playerConfig', 'poster', 'metatags', 'categories'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Video  $video
     * @return \Illuminate\Http\Response
     */
    public function edit(Video $video)
    {
        $metatags = $video->metatags;
        $categories = Category::doesntHave('childrens')->latest()->pluck('name', 'id');
        return view("Backend.videos.edit", compact('video', 'metatags', 'categories'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Video  $video
     * @return \Illuminate\Http\Response
     */
    public function update(Video $video)
    {
        $this->validate($this->request, [
            'title' => ['required', 'max:255'],
            'category_id' => 'numeric|nullable',
            'active' => 'boolean',
            'converter' => 'boolean',
            'is_download' => 'boolean',
            'description' => 'nullable',
            'meta_title' => 'nullable|max:150|min:10',
            'meta_keywords' => 'nullable|max:80|min:8',
            'meta_description' => 'nullable|max:200|min:25',
            'file_url' => 'nullable|exclude_unless:file_path,null|active_url',
            'file_path' => 'nullable|string',
            'thumb_url' => 'nullable|exclude_unless:thumb,null|exclude_unless:thumb_edit,null|active_url',
            'thumb' => 'image|nullable',
            'thumb_edit' => 'nullable',
        ]);

        $this->model = $video;

        $this->model->title = $this->request->input('title');
        $this->model->tags = $this->request->input('tags');
        $this->model->description = $this->request->input('description');
        $this->model->url = ($this->request->boolean('copyfile') && $this->request->anyFilled('file_url', 'file_path')) ? null : $this->request->input('file_url');
        $this->model->is_download = $this->request->boolean('is_download');
        $this->model->featured = $this->request->boolean('featured');
        $this->model->active = ($this->request->boolean('copyfile') && $this->request->anyFilled('file_url', 'file_path')) ? false :  $this->request->boolean('active');
        $this->model->user()->associate($this->request->user());
        $this->model->category()->associate($this->request->input('category_id'));
        $this->model->is_uploaded = ($this->request->boolean('copyfile') && $this->request->anyFilled('file_url', 'file_path')) ? 2 : 1;
        $this->model->published_at = ($this->request->input('published_at')) ? Carbon::createFromFormat("Y-m-d H:i", $this->request->input('published_at')) : now();
        $this->model->save();
        $this->isUpdate = true;
        $this->AddOrUpdateMetags();
        $this->uploadService();
        return redirect()->route('videos.edit', ['video' => $this->model])->with('success', __('message.update', ['name' => 'Video']));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Video  $video
     * @return \Illuminate\Http\Response
     */
    public function destroy(Video $video)
    {
        $this->model = $video;
        $this->model->delete();

        if ($this->request->wantsJson()) {
            return $this->success(['message' => __('message.delete', ['name' => 'Video'])]);
        }

        return redirect()->route('videos.index')->with('success', __('message.delete', ['name' => 'Video']));
    }


    private function uploadService()
    {
        if ($this->request->anyFilled('thumb_edit', 'thumb_url') || $this->request->hasFile('thumb')) {
            if ($this->isUpdate) {
                $this->model->clearMediaCollection(MyConstant::THUMB_COLLECTION);
            }
            $ImageHandler = new UploadService($this->model);
            if ($this->request->filled('thumb_edit')) {
                $ImageHandler->setFrom(UploadService::FILE_FROM_BASE64); //Base64
                $ImageHandler->setBase64($this->request->input('thumb_edit')); //Base64
                $ImageHandler->setExtension("png");
            } else if ($this->request->hasFile('thumb')) {
                $ImageHandler->setFrom(UploadService::FILE_FROM_INPUT); //File
                $ImageHandler->setFile('thumb'); //Thumb
            } else if ($this->request->filled('thumb_url')) {
                $ImageHandler->setFrom(UploadService::FILE_FROM_URL); //URL 
                $ImageHandler->setUrl($this->request->input('thumb_url'), MyConstant::ALLOW_URL_IMAGE_UPLOAD); //URL
            }
            $ImageHandler->setTitle($this->model->title);
            $ImageHandler->setCollection(MyConstant::THUMB_COLLECTION);
            $ImageHandler->save();
        }

        if ($this->request->boolean('copyfile')) {
            if ($this->request->anyFilled('file_url', 'file_path')) {
                if ($this->isUpdate) {
                    $this->model->clearMediaCollection(MyConstant::VIDEO_COLLECTION);
                }
                $pendingVideo = new PendingVideo;
                $pendingVideo->video = $this->model;
                $pendingVideo->isActive = $this->request->boolean('active');

                if ($this->request->filled('file_path')) {
                    $pendingVideo->videoPath = $this->request->input('file_path');
                    $pendingVideo->isVideoUrl = false;
                } else if ($this->request->filled('file_url')) {
                    $pendingVideo->videoPath = $this->request->input('file_url');
                    $pendingVideo->isVideoUrl = true;
                }

                if ($this->request->boolean('converter')) {
                    $pendingVideo->conversion = true;
                    VideoUploadingJob::dispatch($pendingVideo);
                } else {
                    $pendingVideo->conversion = false;
                    VideoUploadingJob::dispatchSync($pendingVideo);
                }
            }
        } else {
            if ($this->isUpdate) {
                $this->model->clearMediaCollection(MyConstant::VIDEO_COLLECTION);
            }
        }
    }


    private function AddOrUpdateMetags()
    {
        if ($this->request->anyFilled('meta_title', 'meta_keywords', 'meta_description')) {
            $this->model->setMetaTags(
                ['metatagable_id' => $this->model->id],
                [
                    'title' => $this->request->input('meta_title'),
                    'keywords' => $this->request->input('meta_keywords'),
                    'description' => $this->request->input('meta_description')
                ]
            );
        }
    }
}
