<?php

namespace App\Http\Controllers\Backend\Videos;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Category;
use App\Traits\Response;
use Str;
use Illuminate\Support\Facades\Validator;
use Cache;
use Alaouy\Youtube\Facades\Youtube;
use App\Jobs\YoutubeVideoCopyToServer;
use Carbon\Carbon;
use Exception;
use Log;
use DataTables;
use Illuminate\Support\Facades\Http;
use App\Models\Video;
use Storage;
use App\Services\UploadService;
use Illuminate\Http\File;
use App\Utils\MyConstant;

class YoutubeUploadController extends Controller
{
    use Response;

    protected Request $request;

    private ?string $query = null;

    private array $part = ['snippet'];

    private int $maxResults = 1;

    private ?string $orderby = null;

    private Video $model;

    public function __construct(Request $request)
    {
        $this->middleware('permission:videos.create');
        $this->request = $request;
    }

    public function index()
    {


        if ($this->request->wantsJson()) {

            $this->query = htmlspecialchars($this->request->query('query'));
            $this->orderby = htmlspecialchars($this->request->query('orderby'));
            $this->maxResults = filter_var($this->request->query('maxresults', 1), FILTER_SANITIZE_NUMBER_INT);
            $cacheList = Str::lower('query_' . $this->query . '_order_' . $this->orderby . '_max_result_' . $this->maxResults);

            $videos = Cache::remember($cacheList, now()->addHour(), function () {
                Log::info('YoutubeController: API Request');
                return $this->getYtVideos();
            });

            return DataTables::of($videos)
                ->addColumn('checkbox', "")
                ->addColumn('thumb', function ($v) {
                    return 'https://i.ytimg.com/vi/' . $v['id'] . '/mqdefault.jpg';
                })
                ->make(true);
        }

        $yt_sorting = [
            'relevance' => 'Relevance',
            'date' => 'Date',
            'rating' => 'Rating',
            'title' => 'Title',
            'viewCount' => 'View Count',
        ];

        $categories = Category::doesntHave('childrens')->latest()->pluck('name', 'id');
        return view('Backend.videos.youtube', compact('categories', 'yt_sorting'));
    }

    public function upload()
    {
        $validator = Validator::make($this->request->all(), [
            'title' => ['required', 'max:255'],
            'youtube_id' => 'required',
            'category_id' => 'numeric|nullable',
        ]);

        if ($validator->fails()) {
            return $this->failure(['message' => $validator->errors()->__toString()]);
        }

        $ytVideoUrl = sprintf("https://www.youtube.com/watch?v=%s", $this->request->input('youtube_id'));

        $this->model = new Video;
        $this->model->title = $this->request->input('title');
        $this->model->is_download = $this->request->boolean('is_download');
        $this->model->featured = $this->request->boolean('featured');
        $this->model->url = ($this->request->boolean('copyfile')) ? null : $ytVideoUrl;
        $this->model->youtube_id = $this->request->input('youtube_id');
        $this->model->active = ($this->request->boolean('copyfile')) ? false : $this->request->boolean('active');
        $this->model->published_at = ($this->request->input('published_at')) ? Carbon::createFromFormat("Y-m-d H:i", $this->request->input('published_at')) : now();
        $this->model->is_uploaded = $this->request->boolean('copyfile') ? 2 : 1;
        $this->model->user()->associate($this->request->user());
        $this->model->category()->associate($this->request->input('category_id'));
        $this->model->save();


        $ytThumbCopy = false;
        $ytThumbToSave = 'tmp/' . $this->request->input('youtube_id') . '.jpg';
        $ytThumb = sprintf("https://i.ytimg.com/vi/%s/%s.jpg", $this->request->input('youtube_id'), "maxresdefault");
        $ytThumbResponse = Http::get($ytThumb);
        if ($ytThumbResponse->ok()) {
            Storage::disk('local')->put($ytThumbToSave, $ytThumbResponse->body());
            $ytThumbCopy = true;
        } else if ($this->request->filled('thumb_url')) {
            $ytThumbResponse = Http::get($this->request->input('thumb_url'));
            if ($ytThumbResponse->ok()) {
                Storage::disk('local')->put($ytThumbToSave, $ytThumbResponse->body());
                $ytThumbCopy = true;
            }
        } else {
            info("YoutubeUploadContoller: thumbnail are not found " . $this->request->input('youtube_id'));
        }

        if ($ytThumbCopy) {
            $ImageHandler = new UploadService($this->model);
            $ImageHandler->setFrom(UploadService::FILE_ADD_FROM_FILE);
            $ImageHandler->setFile(new File(Storage::disk('local')->path($ytThumbToSave)));
            $ImageHandler->setTitle($this->model->title);
            $ImageHandler->setCollection(MyConstant::THUMB_COLLECTION);
            $ImageHandler->save();
        }


        if ($this->request->boolean('copyfile')) {
            $quality = $this->request->input('quality', 'medium');
            $status =  $this->request->boolean('active');
            dispatch(new YoutubeVideoCopyToServer($this->model, $status, $quality, $this->request->boolean('converter')));
        }

        return $this->success();
    }

    /**
     * get YouTube Videos
     *
     * @return Collection
     */
    private function getYtVideos()
    {
        $YtVideos = array();
        $singleVideo = false;
        $isPlaylist = false;
        if ($this->query) {
            try {
                $YT_API_KEY = setting('youtube_key') ?: config('youtube.key');
                Youtube::setApiKey($YT_API_KEY);
                if ($videoid = $this->getVideoId($this->query)) {
                    $YtVideos[] = Youtube::getVideoInfo($videoid, $this->part);
                    $singleVideo = true;
                } else if ($channel = $this->getChannelIdOrName($this->query)) {
                    $channelUid = null;
                    if ($channel['type'] == "channel") {
                        $channelUid = $channel['id'];
                    } else if ($channel['type'] == 'user' || $channel['type'] == 'c') {
                        $channelInfo = Youtube::getChannelByName($channel['id'], [], ['id']);
                        $channelUid = $channelInfo->id;
                    }
                    if ($channelUid)
                        $YtVideos = Youtube::listChannelVideos($channelUid, $this->maxResults, $this->orderby, $this->part);
                } elseif ($playlistId = $this->getPlayListId($this->query)) {
                    $playlistResults = Youtube::getPlaylistItemsByPlaylistId($playlistId, '', $this->maxResults, $this->part);
                    if ($playlistResults) {
                        $YtVideos = $playlistResults['results'];
                    }
                    $isPlaylist = true;
                } else {
                    $params = [
                        'q' => $this->query,
                        'type' => 'video',
                        'part' => implode(',', $this->part),
                        'order' => $this->orderby,
                        'maxResults' => $this->maxResults,
                    ];
                    $YtVideos = Youtube::searchAdvanced($params);
                }
            } catch (Exception $e) {
                Log::info("YoutubeVideoController: " . $e->getMessage());
            }
        }

        $Videos = collect();
        foreach ($YtVideos as $vid) {
            if ($singleVideo) {
                $ytVidId = $vid->id;
                $ownerchannel = ['id' => $vid->snippet->channelId, 'name' => $vid->snippet->channelTitle];
            } else if ($isPlaylist) {
                $ytVidId = $vid->snippet->resourceId->videoId;
                $ownerchannel = ['id' => $vid->snippet->videoOwnerChannelTitle, 'name' => $vid->snippet->videoOwnerChannelTitle];
            } else {
                $ytVidId = $vid->id->videoId;
                $ownerchannel = ['id' => $vid->snippet->channelId, 'name' => $vid->snippet->channelTitle];
            }

            $vidData = [
                'id' => $ytVidId,
                'title' => $vid->snippet->title,
                'thumbnail' => collect($vid->snippet->thumbnails)->last()->url,
                'channel' => $ownerchannel,
                'description' => $vid->snippet->description,
                'publishedAt' => Carbon::parse($vid->snippet->publishedAt)->format('d M, y H:i')
            ];
            $Videos = $Videos->add($vidData);
        }

        return $Videos;
    }


    private function getVideoId($str)
    {
        if (strpos($str, 'list=') !== false) {
            return false;
        }
        if (@preg_match('%(?:youtube(?:-nocookie)?\.com/(?:[^/]+/.+/|(?:v|e(?:mbed)?)/|.*[?&]v=)|youtu\.be/)([^"&?/ ]{11})%i', $str, $match)) {
            return $match[1];
        }
        return false;
    }

    private function getChannelIdOrName($str)
    {
        $regex = '/\/(channel|user|c)\/([^\/]+)$/m';
        if (preg_match($regex, $str, $match)) {
            return ['type' => $match[1], 'id' => $match[2]];
        }
        return false;
    }

    private function getPlayListId($str)
    {
        if (strpos($str, 'list=') !== false) {
            $params = Youtube::_parse_url_query($str);
            if ($params) {
                return $params['list'];
            }
        }
        return false;
    }
}
