<?php

namespace App\Http\Controllers\Frontend;

use App\Http\Controllers\Controller;
use App\Http\Resources\CategoryResource;
use App\Http\Resources\VideoResource;
use  App\Utils\FrontConst;
use App\Models\Category;
use App\Traits\Response;
use App\Models\Video;
use Butschster\Head\Facades\Meta;
use MyConstant;

class CategoryController extends Controller
{
    use Response;

    private $data =  null;

    private string $sort;

    private ?Category $category = null;

    private $meta_og;


    public function __construct()
    {

        $this->meta_og = (new \Butschster\Head\Packages\Entities\OpenGraphPackage('meta_og'))
            ->setUrl(url()->current());
    }

    /**
     * get Category Videos
     *
     * @param string $sort
     * @return View
     */

    public function category()
    {
        $this->sort = request('sort', FrontConst::SORT_LATEST);
        $this->getCategories();
        if (request()->wantsJson() && isAjaxPaging()) {
            return $this->success([
                'data' => [
                    'categories' => CategoryResource::collection($this->data),
                    'pagination' => ['totalPage' => $this->data->lastPage(), 'page' => $this->data->currentPage()],
                ]
            ]);
        }
        $this->setMetas();
        return view('Frontend.Categories.category', [
            'data' => $this->data,
            'filters' => ['route' => 'category.all', 'filters' => Category::getFilters()],
            'sort' => $this->sort,
        ]);
    }

    /**
     * get Subcategory
     *
     * @param string $slug
     * @param string $sort
     * @return View
     */
    public function subCategory($slug)
    {
        $this->sort = request('sort', FrontConst::SORT_LATEST);
        $this->getCategory($slug);
        if ($this->category->videos_count !== 0) {
            return redirect()->route('categoryvideos.all', ['slug' => $this->category->slug]);
        }
        $this->getCategories();
        if (request()->wantsJson() && isAjaxPaging()) {
            return $this->success([
                'data' => [
                    'categories' => CategoryResource::collection($this->data),
                    'pagination' => ['totalPage' => $this->data->lastPage(), 'page' => $this->data->currentPage()],
                ]
            ]);
        }
        $this->setSubMetas();
        return view('Frontend.Categories.subcategory', [
            'category' => $this->category,
            'data' => $this->data,
            'filters' => ['route' => 'subcategory.all', 'filters' => Category::getFilters(['slug' => $slug])],
            'sort' => $this->sort,
        ]);
    }

    /**
     * get Category Videos
     *
     * @param string $slug
     * @param string $sort
     * @return View
     */
    public function categoryVideos($slug)
    {
        $this->sort = request('sort', FrontConst::SORT_LATEST);
        $this->getCategory($slug);

        if ($this->category->childrens_count !== 0) {
            return redirect()->route('subcategory.all', ['slug' => $this->category->slug]);
        }

        $this->getVideos();
        if ($media = $this->category->getFirstMedia(MyConstant::THUMB_COLLECTION)) {
            $bgPlaceHolder = 'url(' . optional($media)->responsiveImages(MyConstant::THUMB)->getPlaceholderSvg() . ')';
        }
        if (request()->wantsJson() && isAjaxPaging()) {
            return $this->success([
                'data' => [
                    'videos' => VideoResource::collection($this->data),
                    'pagination' => ['totalPage' => $this->data->lastPage(), 'page' => $this->data->currentPage()],
                ]
            ]);
        }
        $this->setCategoryMetas();
        return view('Frontend.Categories.videos', [
            'category' => $this->category,
            'data' => $this->data,
            'filters' => ['route' => 'categoryvideos.all', 'filters' => Video::getFilters(['slug' => $slug])],
            'sort' => $this->sort,
            'bgPlaceHolder' => $bgPlaceHolder ?? 'linear-gradient( 135deg, #5f3bc5 10%, #000 100%)',
        ]);
    }

    private function getCategories()
    {
        $perPage = setting(FrontConst::LIMIT_PER_PAGE_CATEGORIES, FrontConst::DEFAULT_CATEGORIES_PER_PAGE);
        if ($this->category)
            $data = $this->category->childrens()->active()->select(FrontConst::CATEGORY_ITEMS_COLUMNS);
        else {
            $data  = Category::select(FrontConst::CATEGORY_ITEMS_COLUMNS)->parents()->active();
        }
        $this->data = $data->withCount(["childrens", 'videos'])->customSort($this->sort)->paginate($perPage)->withQueryString();
        $this->data->load(['media']);
    }

    private function getCategory(string $slug)
    {
        $this->category = cache()->remember(FrontConst::CACHE['category'] . $slug, FrontConst::getCacheExpire(), function () use ($slug) {
            return Category::active()->with('metatags')->withCount("childrens", 'videos')->WhereSlug($slug)->firstOrFail();
        });
    }


    private function getVideos()
    {
        $perPage = setting(FrontConst::LIMIT_PER_PAGE_VIDEOS, FrontConst::DEFAULT_VIDEOS_PER_PAGE);
        $videos = $this->category->videos()->active()->customSort($this->sort)->select(FrontConst::VIDEO_ITEMS_COLUMNS);
        $this->data = $videos->paginate($perPage)->withQueryString();
        $this->data->load(['media']);
    }

    private function setMetas()
    {

        $title = trans('meta_tags.categories.title');
        $desc = trans('meta_tags.categories.description');
        Meta::prependTitle($title);
        Meta::setDescription($desc);

        if ($keywords = trans('meta_tags.categories.keywords')) {
            Meta::setKeywords($keywords);
        }


        $this->meta_og->setType('article')
            ->setTitle($title)
            ->setDescription($desc)
            ->addImage(asset(FrontConst::META_OG_IMAGE));

        Meta::registerPackage($this->meta_og);
    }
    private function setSubMetas()
    {

        $title = $this->category->meta_title ?: trans('meta_tags.subcategories.title', ['title' => $this->category->name]);
        $desc  = $this->category->meta_description ?: trans('meta_tags.subcategories.description', ['title' => $this->category->name]);

        Meta::prependTitle($title);
        Meta::setDescription($desc);
        if ($keywords = $this->category->meta_keywords ?: trans('meta_tags.subcategories.keywords', ['title' => $this->category->name])) {
            Meta::setKeywords($keywords);
        }
        $this->meta_og->setType('article')
            ->setTitle($title)
            ->setDescription($desc)
            ->addImage($this->category->getFirstMediaUrl(MyConstant::THUMB_COLLECTION, MyConstant::THUMB) ?: asset(FrontConst::META_OG_IMAGE));

        Meta::registerPackage($this->meta_og);
    }
    private function setCategoryMetas()
    {

        $title = $this->category->meta_title ?: trans('meta_tags.category_videos.title', ['title' => $this->category->name]);
        $desc = $this->category->meta_description ?: trans('meta_tags.category_videos.description', ['title' => $this->category->name]);

        Meta::prependTitle($title);
        Meta::setDescription($desc);

        if ($keywords = $this->category->meta_keywords ?: trans('meta_tags.category_videos.keywords', ['title' => $this->category->name])) {
            Meta::setKeywords($keywords);
        }

        $this->meta_og->setType('article')
            ->setTitle($title)
            ->setDescription($desc)
            ->addImage($this->category->getFirstMediaUrl(MyConstant::THUMB_COLLECTION, MyConstant::THUMB) ?: asset(FrontConst::META_OG_IMAGE));


        Meta::registerPackage($this->meta_og);
    }
}
