<?php

namespace App\Http\Controllers\Frontend;

use App\Http\Controllers\Controller;
use App\Http\Middleware\RedirectIfNotInstalled;
use App\Models\Playlist;
use App\Models\Video;
use App\Utils\FrontConst;
use Butschster\Head\Facades\Meta;
use App\Models\Category;
use App\Models\Page;
use Illuminate\Http\Request;
use Str;

class HomeController extends Controller
{


    public function __construct()
    {
        $this->middleware(RedirectIfNotInstalled::class);
    }

    public function index()
    {

        $title = setting('meta_title', config('app.name'));
        Meta::setTitle($title);
        Meta::includePackages("jquery-ui");
        Page::getAllList();


        $meta_og = (new \Butschster\Head\Packages\Entities\OpenGraphPackage('meta_og'))
            ->setUrl(url()->current())
            ->setType('website')
            ->setTitle($title)
            ->setDescription(setting('meta_description', config('app.name')))
            ->addImage(asset(FrontConst::META_OG_IMAGE));

        Meta::registerPackage($meta_og);

        if (setting(FrontConst::UI) === "ui_simple") {
            return $this->simpleUi();
        }
        return $this->defaultUi();
    }

    public function pageInfo(Request $reuquest, $slug)
    {
        $page = Page::where('slug', $slug)->where('active', 1)->firstOrFail();
        Meta::prependTitle($page->title);
        return view('Frontend.Home.page', ['page' => $page]);
    }

    private function defaultUi()
    {

        $trendingVideos = cache()->remember(FrontConst::CACHE['home_trending'], FrontConst::getCacheExpire(), function () {
            $videos = Video::trending()->active()->limit(setting(FrontConst::LIMIT_HOME_TREND, FrontConst::LIMIT_DEFAULT_HOME))->get(FrontConst::VIDEO_ITEMS_COLUMNS);
            $videos->load(FrontConst::MODEL_MEDIA);
            return $videos;
        });

        $exceptVideos = $trendingVideos->modelKeys();

        $latestVideos = cache()->remember(FrontConst::CACHE['home_latest'], FrontConst::getCacheExpire(), function () use ($exceptVideos) {
            $videos = Video::latest("published_at")->whereKeyNot($exceptVideos)->active()->limit(setting(FrontConst::LIMIT_HOME_LATEST, FrontConst::LIMIT_DEFAULT_HOME))->get(FrontConst::VIDEO_ITEMS_COLUMNS);
            $videos->load(FrontConst::MODEL_MEDIA);
            return $videos;
        });


        $latestPlaylist = cache()->remember(FrontConst::CACHE['home_playlist'], FrontConst::getCacheExpire(), function () {
            $playlists = Playlist::latest()->active()->withCount(['videos'])->limit(setting(FrontConst::LIMIT_HOME_PLAYLIST, FrontConst::LIMIT_DEFAULT_HOME))->get(FrontConst::PLAYLIST_ITEMS_COLUMNS);
            $playlists->load([FrontConst::MODEL_MEDIA]);
            return $playlists;
        });

        $categories = cache()->remember(FrontConst::CACHE['home_categories'], FrontConst::getCacheExpire(), function () {
            $categories = Category::select(FrontConst::CATEGORY_ITEMS_COLUMNS)->parents()->active()->withCount(["childrens", 'videos'])->limit(setting(FrontConst::LIMIT_HOME_CATEGORY, FrontConst::LIMIT_DEFAULT_HOME))->get(FrontConst::CATEGORY_ITEMS_COLUMNS);
            $categories->load([FrontConst::MODEL_MEDIA]);
            return $categories;
        });

        return view('Frontend.Home.index', compact('trendingVideos', 'categories', 'latestVideos', 'latestPlaylist'));
    }

    private function simpleUi()
    {


        $latestVideos = cache()->remember(FrontConst::CACHE['home_latest'], FrontConst::getCacheExpire(), function () {
            $videos = Video::latest("published_at")->active()->limit(setting(FrontConst::LIMIT_HOME_LATEST, FrontConst::LIMIT_DEFAULT_HOME))->get(FrontConst::VIDEO_ITEMS_COLUMNS);
            $videos->load(FrontConst::MODEL_MEDIA);
            return $videos;
        });


        $data = [
            'videos' => $latestVideos,
            'filters' => ['route' => 'videos.all', 'filters' => Video::getFilters()],
            'sort' => FrontConst::SORT_LATEST,
        ];

        return view('Frontend.Home.simple-ui', $data);
    }

    public function getSearchIndex()
    {
        $q = request('q');
        if (strlen($q) < 2) {
            return response()->json(['data' => []]);
        }
        $q = str::lower($q);
        $videos = Video::where("title", "LIKE", '%' . $q . '%')->orWhere("tags", "LIKE", "%" . $q . "%")->get("title", "tags");

        $indexs = [];
        foreach ($videos as $item) {
            $title = \Str::of($item->title)->remove([',', '-', '|', '/', '&']);
            $tags = Str::of($item->tags)->explode(",");
            foreach ($tags as $tag) {
                array_push($indexs, $tag);
            }
            array_push($indexs, $title->title()->lower());
            $titles = $title->explode(' ');
            $totalwords = count($titles);
            if ($totalwords > 3) {
                $i = 0;
                $j = 0;
                while ($j < $totalwords) {
                    array_push($indexs, Str::lower($titles[$i] . ' ' . $titles[$i + 1] . ' ' . $titles[$i + 2] . ' ' . $titles[$i + 3]));
                    $j = $j + 4;
                    $i++;
                }
            }
        }

        $data = collect($indexs)->reject(function ($v) use ($q) {
            return (strpos($v, $q) !== 0);
        })->unique();

        return response()->json(['data' => $data]);
    }
}
