<?php

namespace App\Http\Controllers\Frontend;

use App\Http\Controllers\Controller;
use App\Http\Resources\PlaylistResource;
use App\Http\Resources\VideoResource;
use App\Utils\FrontConst;
use App\Traits\Response;
use App\Models\Playlist;
use Butschster\Head\Facades\Meta;
use MyConstant;

class PlaylistController extends Controller
{
    use Response;


    private  $data =  null;

    private Playlist $playlist;

    private $meta_og;

    private string $sort;


    public function __construct()
    {

        $this->meta_og = (new \Butschster\Head\Packages\Entities\OpenGraphPackage('meta_og'))
            ->setUrl(url()->current());
    }

    /**
     * get Playlists
     *
     * @param string $sort
     * @return View
     */
    public function playlist()
    {
        $this->sort = request('sort', FrontConst::SORT_LATEST);
        $this->getPlaylists();
        if (request()->wantsJson() && isAjaxPaging()) {
            return $this->success([
                'data' => [
                    'playlists' => PlaylistResource::collection($this->data),
                    'pagination' => ['totalPage' => $this->data->lastPage(), 'page' => $this->data->currentPage()],

                ],
            ]);
        }
        $this->setMetas();
        return view('Frontend.Playlists.playlist', [
            'data' => $this->data,
            'filters' => ['route' => 'playlists.all', 'filters' => Playlist::getFilters()],
            'sort' => $this->sort,
        ]);
    }


    /**
     * get Playlist Videos
     *
     * @param string $slug
     * @param string $sort
     * @return View
     */
    public function playlistVideos($slug)
    {
        $this->sort = request('sort', FrontConst::SORT_DEFAULT_PLAYLIST_VIDEOS);
        $this->getPlaylist($slug);
        $this->getVideos();

        if ($media = $this->playlist->getFirstMedia(MyConstant::THUMB_COLLECTION)) {
            $bgPlaceHolder = 'url(' . optional($media)->responsiveImages(MyConstant::THUMB)->getPlaceholderSvg() . ')';
        }

        if (request()->wantsJson() && isAjaxPaging()) {
            return $this->success([
                'data' => [
                    'videos' => VideoResource::collection($this->data),
                    'pagination' => ['totalPage' => $this->data->lastPage(), 'page' => $this->data->currentPage()],
                ]
            ]);
        }
        $this->setPlaylistMetas();
        return view('Frontend.Playlists.videos', [
            'playlist' => $this->playlist,
            'data' => $this->data,
            'filters' => ['route' => 'playlistvideos.all', 'filters' => Playlist::getFilters(['slug' => $slug])],
            'sort' => $this->sort,
            'bgPlaceHolder' => $bgPlaceHolder ?? 'linear-gradient( 135deg, #5f3bc5 10%, #000 100%)',
        ]);
    }

    private function getPlaylists()
    {
        $perPage = setting(FrontConst::LIMIT_PER_PAGE_PLAYLISTS, FrontConst::DEFAULT_PLAYLISTS_PER_PAGE);
        $data = Playlist::select(FrontConst::PLAYLIST_ITEMS_COLUMNS)->active();
        $this->data = $data->withCount(['videos'])->customSort($this->sort)->paginate($perPage)->withQueryString();
        $this->data->load(['media']);
    }

    private function getPlaylist(string $slug)
    {

        $this->playlist =  cache()->remember(FrontConst::CACHE['playlist'] . $slug, FrontConst::getCacheExpire(), function () use ($slug) {
            return Playlist::active()->with(['metatags', 'media'])->WhereSlug($slug)->firstOrFail();
        });
    }

    private function getVideos()
    {
        $perPage = setting(FrontConst::LIMIT_PER_PAGE_VIDEOS, FrontConst::DEFAULT_VIDEOS_PER_PAGE);
        $videos = $this->playlist->videos()->active();
        switch ($this->sort) {
            case FrontConst::SORT_LATEST:
                $videos->orderByPivot("updated_at", "desc");
                break;
            case FrontConst::SORT_AZ:
                $videos->orderBy('title');
                break;
            case FrontConst::SORT_ZA:
                $videos->orderByDesc('title');
                break;
            case FrontConst::SORT_OLDEST:
                $videos->orderByPivot("updated_at");
                break;
            default:
                $videos->orderByPivot("updated_at");
                break;
        }
        $this->data = $videos->select(FrontConst::VIDEO_ITEMS_COLUMNS)->paginate($perPage)->withQueryString();
        $this->data->load(['media']);
    }

    private function setMetas()
    {
        $title = trans('meta_tags.playlists.title');
        $desc = trans('meta_tags.playlists.description');

        Meta::prependTitle($title);
        Meta::setDescription($desc);
        if ($keywords = trans('meta_tags.playlists.keywords')) {
            Meta::setKeywords($keywords);
        }

        $this->meta_og->setType('article')
            ->setTitle($title)
            ->setDescription($desc)
            ->addImage(asset(FrontConst::META_OG_IMAGE));

        Meta::registerPackage($this->meta_og);
    }
    private function setPlaylistMetas()
    {

        $title = $this->playlist->meta_title ?: trans('meta_tags.playlist_videos.title', ['title' => $this->playlist->name]);
        $desc = $this->playlist->meta_description ?: trans('meta_tags.playlist_videos.description', ['title' => $this->playlist->name]);

        Meta::prependTitle($title);
        Meta::setDescription($desc);

        if ($keywords = $this->playlist->meta_keywords ?: trans('meta_tags.playlist_videos.keywords', ['title' => $this->playlist->name])) {
            Meta::setKeywords($keywords);
        }

        $this->meta_og->setType('article')
            ->setTitle($title)
            ->setDescription($desc)
            ->addImage($this->playlist->getFirstMediaUrl(MyConstant::THUMB_COLLECTION, MyConstant::THUMB) ?: asset(FrontConst::META_OG_IMAGE));

        Meta::registerPackage($this->meta_og);
    }
}
