<?php

namespace App\Http\Controllers\Frontend;

use App\Events\VideoViewed;
use App\Http\Controllers\Controller;
use App\Http\Resources\VideoResource;
use App\Models\Playlist;
use App\Utils\FrontConst;
use App\Models\Video;
use App\Models\VideoView;
use App\Traits\Response;
use MyConstant;
use Str;
use Butschster\Head\Facades\Meta;
use Cache;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\URL;
use Spatie\MediaLibrary\MediaCollections\Models\Media;

class VideoController extends Controller
{

    use Response;

    private $data =  null;

    private Video $video;

    private string $sort;

    private ?string $query = null;

    private $meta_og;

    private ?string $contentUri = null;

    private ?array $schema = null;

    public function __construct()
    {

        $this->meta_og = (new \Butschster\Head\Packages\Entities\OpenGraphPackage('meta_og'))
            ->setUrl(url()->current());
    }



    /**
     * Get VIdeos List
     *
     * @param string $sort
     * @return View
     */
    public function videos()
    {
        $this->sort = request('sort', FrontConst::SORT_VIDEO_DEFAULT);
        $this->getVideos();
        if (request()->wantsJson() && isAjaxPaging()) {
            return $this->success([
                'data' => [
                    'videos' => VideoResource::collection($this->data),
                    'pagination' => ['totalPage' => $this->data->lastPage(), 'page' => $this->data->currentPage()],
                ]
            ]);
        }
        $this->setMetas();
        return view('Frontend.Videos.videos', [
            'data' => $this->data,
            'filters' => ['route' => 'videos.all', 'filters' => Video::getFilters()],
            'sort' => $this->sort
        ]);
    }

    /**
     * get Top Videos
     *
     * @param string $sort
     * @return View
     */
    public function top()
    {
        $this->sort = request('sort', FrontConst::SORT_TODAY);
        $this->getTopVideos();
        if (request()->wantsJson() && isAjaxPaging()) {
            return $this->success([
                'data' => [
                    'videos' => VideoResource::collection($this->data),
                    'pagination' => ['totalPage' => $this->data->lastPage(), 'page' => $this->data->currentPage()],
                ]
            ]);
        }

        $this->setTopMetas();;
        return view('Frontend.Videos.top', [
            'data' => $this->data,
            'filters' => ['route' => 'topvideos.all', 'filters' => VideoView::getFilters()],
            'sort' => $this->sort,
        ]);
    }


    /**
     * get Search Videos
     *
     * @param string $sort
     * @return View
     */
    public function searchVideos()
    {
        $this->sort = request('sort', FrontConst::SORT_AZ);
        $this->query = request()->input('q');
        $this->getSearchVideos();
        if (request()->wantsJson() && isAjaxPaging()) {
            return $this->success([
                'data' => [
                    'q' => $this->query,
                    'videos' => VideoResource::collection($this->data),
                    'pagination' => ['totalPage' => $this->data->lastPage(), 'page' => $this->data->currentPage()],
                ]
            ]);
        }

        $this->setSearchMetas();
        return view('Frontend.Videos.search', [
            'q' => $this->query,
            'data' => $this->data,
            'filters' => ['route' => 'search.videos', 'filters' => Video::getFilters(['q' => $this->query])],
            'sort' => $this->sort
        ]);
    }

    /**
     * get Tagged Videos
     *
     * @param string $slug
     * @param string $sort
     * @return View
     */
    public function tagVideos($slug)
    {
        $this->sort = request('sort', FrontConst::SORT_VIDEO_DEFAULT);
        $this->query = Str::of($slug)->replace("-", " ")->title();
        $this->getTagVideos();
        if (request()->wantsJson() && isAjaxPaging()) {
            return $this->success([
                'data' => [
                    'videos' => VideoResource::collection($this->data),
                    'pagination' => ['totalPage' => $this->data->lastPage(), 'page' => $this->data->currentPage()],
                ]
            ]);
        }

        $this->setTagMetas();
        return view('Frontend.Videos.tag', [
            'title' => $this->query,
            'data' => $this->data,
            'filters' => ['route' => 'tags.all', 'filters' => Video::getFilters(['slug' => $slug])],
            'sort' => $this->sort,
        ]);
    }

    /**
     * Single Video page function
     *
     * @param string $slug
     * @return view
     */
    public function video($slug)
    {
        $this->getVideo($slug);

        event(new VideoViewed($this->video->id));

        $this->getReletedVideos();

        $poster = optional($this->video->getFirstMedia(MyConstant::THUMB_COLLECTION))->getUrl(MyConstant::THUMB);

        if (!$poster) {
            $poster = asset('images/default.jpg');
        }

        //Player Config
        $playerConfig = [
            'ratio' => "16:9",
            "loadSprite" => true,
            "iconUrl" => asset('static/plyr.svg'),
            "blankVideo" => asset('static/blank.mp4'),
            "quality" => FrontConst::PLAYER_DEFAULT_QUALITY
        ];

        if (setting('player_ad_tagurl')) {
            $playerConfig['ads'] = [
                'enabled' => true,
                'publisherId' => setting('ad_publisher_id'),
                'tagUrl' => setting('player_ad_tagurl'),
            ];
        }


        $sources = [];

        if ($this->video->play_urls->isEmpty() && ($embed = $this->video->getEmbedId())) {
            $sources[] = [
                'src' => $embed['id'],
                'provider' => $embed['provider'],
            ];
        } else {
            $sources = $this->video->play_urls;
            $this->contentUri = optional($sources->first())["src"];
        }

        $player_data = [
            'title' => $this->video->title,
            'type' => "video",
            'sources' => $sources,
            'poster' => $poster,
        ];

        $this->meta_og->addImage($poster);

        $this->setVideoMetas();
        return view("Frontend.Videos.video", [
            'poster' => $poster,
            'video' => $this->video,
            'reletedvideos' => $this->data,
            'playerConfig' => $playerConfig,
            'playlists' => $this->getPlaylists(),
            'player_data' => $player_data,
            'schema' => $this->schema
        ]);
    }

    public function getLinks($slug)
    {
        $this->getVideo($slug);
        $sources = [];

        if ($this->video->play_urls->isEmpty() && ($embed = $this->video->getEmbedId())) {
            $sources[] = [
                'src' => $embed['id'],
                'provider' => $embed['provider'],
            ];
        } else {
            $sources = $this->video->play_urls;
        }

        $player_data = [
            'title' => $this->video->title,
            'type' => "video",
            'sources' => $sources,
            'poster' => $this->video->getFirstMediaUrl(MyConstant::THUMB_COLLECTION, MyConstant::THUMB) ?: asset('images/default.jpg'),
        ];

        return $this->success(['data' => [
            'source' => $player_data,
        ]]);
    }

    public function downloads($slug)
    {
        $this->getVideo($slug);

        if (!$this->video->is_download || $this->video->url) {
            return $this->failure();
        }

        $sources = $this->video->downloads_files->map(function ($item) {
            $label = ($item['size']) ? $item['size'] . 'p' : "Orginal";
            return ['url' => URL::signedRoute("video.download", ['uuid' => $item['uuid']]), 'label' => $label . ' - ' . $item['filesize']];
        });

        if ($sources->isNotEmpty()) {
            return $this->success(['data' => $sources]);
        }

        return $this->failure();
    }


    public function download(string $uuid)
    {
        $media = Media::findByUuid($uuid);
        abort_unless($media, 404);
        $quality = $media->getCustomProperty('height');
        $quality = $quality ? $quality . 'p' : "Orginal";
        $filename = $media->name . " -  " .  $quality . '.' . pathinfo($media->file_name, PATHINFO_EXTENSION);
        return response()->download($media->getPath(), $filename);
    }

    public function videoLike(Request $request, $slug)
    {
        $this->getVideo($slug);
        if ($request->input('unlike') == true) {
            $this->decrementUnLikes();
        }
        if ($request->input('like') == true) {
            $this->video->increment("likes");
        } else {
            $this->video->decrementLikes();
        }
        Cache::forget(FrontConst::CACHE['video_details'] . $slug);
        return $this->success(['data' => ['likes' => convertShorten($this->video->likes)]]);
    }

    public function videoUnlike(Request $request, $slug)
    {
        $this->getVideo($slug);
        if ($request->input('like') == true) {
            $this->decrementLikes();
        }
        if ($request->input('unlike') == true) {
            $this->video->increment("unlikes");
        } else {
            $this->decrementUnLikes();
        }
        Cache::forget(FrontConst::CACHE['video_details'] . $slug);
        return $this->success(['data' => ['likes' => convertShorten($this->video->likes)]]);
    }

    private function decrementLikes()
    {
        if ($this->video->likes != 0) {
            $this->video->decrement("likes");
        }
    }

    private function decrementUnLikes()
    {
        if ($this->video->unlikes != 0) {
            $this->video->decrement("unlikes");
        }
    }



    /**
     * more Releted if videos
     *
     * @param string $slug
     * @return \Illuminate\Http\JsonResponse
     */
    public function moreReletedVideos($slug)
    {
        abort_if(!request()->wantsJson(), 404);
        $page = request('page', 1);
        $this->getVideo($slug);
        $perPage = setting(FrontConst::LIMIT_PER_PAGE_VIDEOS, FrontConst::DEFAULT_VIDEOS_PER_PAGE);
        $previouslyShowed = setting(FrontConst::LIMIT_VIDEO_RELETED, FrontConst::LIMIT_DEFAULT_VIDEO_RELETED);
        $offset = ($page * $perPage) - $perPage;
        $offset = $offset + $previouslyShowed;
        $videos = Video::active()->whereKeyNot($this->video->getKey())->reletedvideos($this->getVideoTags());
        $totalItems = $videos->count();
        $this->data =  $videos->orderByDesc('published_at')->orderByDesc("today_views")->skip($offset)->take($perPage)->get(FrontConst::VIDEO_ITEMS_COLUMNS);
        $this->data->load('media');
        $totalPage = intval(ceil(($totalItems - $previouslyShowed)  / $perPage));
        return $this->success([
            'data' => [
                'reletedvideos' => VideoResource::collection($this->data),
                'pagination' => ['totalPage' => $totalPage, 'page' => $page],
            ]
        ]);
    }


    private function getVideos()
    {
        $perPage = setting(FrontConst::LIMIT_PER_PAGE_VIDEOS, FrontConst::DEFAULT_VIDEOS_PER_PAGE);
        $videos = Video::active()->customSort($this->sort)->select(FrontConst::VIDEO_ITEMS_COLUMNS);
        $this->data = $videos->paginate($perPage)->withQueryString();
        $this->data->load(['media']);
    }

    private function getTagVideos()
    {
        $perPage = setting(FrontConst::LIMIT_PER_PAGE_VIDEOS, FrontConst::DEFAULT_VIDEOS_PER_PAGE);
        $videos = Video::active()->where('tags', 'LIKE', '%' . $this->query . '%')->customSort($this->sort)->select(FrontConst::VIDEO_ITEMS_COLUMNS);
        $this->data = $videos->paginate($perPage)->withQueryString();
        $this->data->load(['media']);
    }

    private function getSearchVideos()
    {
        $perPage = setting(FrontConst::LIMIT_PER_PAGE_VIDEOS, FrontConst::DEFAULT_VIDEOS_PER_PAGE);
        $videos = Video::active()
            ->where(function ($query) {
                $query->where('title', 'LIKE', '%' . $this->query . '%')
                    ->orWhere('description', 'LIKE', '%' . $this->query . '%')
                    ->orWhere('tags', 'LIKE', '%' . $this->query . '%');
            })->customSort($this->sort)->select(FrontConst::VIDEO_ITEMS_COLUMNS);

        $this->data = $videos->paginate($perPage)->withQueryString();
        $this->data->load(['media']);
    }

    private function getTopVideos()
    {
        $isToday = false;
        switch ($this->sort) {
            case FrontConst::SORT_YESTERDAY:
                $startDate = now()->yesterday();
                break;
            case FrontConst::SORT_WEEK:
                $startDate = now()->subWeek();
                break;
            case FrontConst::SORT_MONTH:
                $startDate = now()->subMonth();
                break;
            case FrontConst::SORT_SEMI:
                $startDate = now()->subMonths(6);
                break;
            case FrontConst::SORT_ANNUAL:
                $startDate = now()->subYear();
                break;
            default:
                $isToday = true;
                break;
        }
        $perPage = setting(FrontConst::LIMIT_PER_PAGE_VIDEOS, FrontConst::DEFAULT_VIDEOS_PER_PAGE);
        $videos = Video::active();

        if (!$isToday) {
            $videoview = new VideoView;
            $videos->join($videoview->getTable(), $videoview->qualifyColumn('video_id'), '=', $videos->qualifyColumn('id'))
                ->whereBetween($videoview->qualifyColumn('created_at'), [$startDate, now()])->orderByDesc($videoview->qualifyColumn('views'));
        } else {
            $videos->orderByDesc("today_views");
        }
        $this->data = $videos->select($videos->qualifyColumns(FrontConst::VIDEO_ITEMS_COLUMNS))->paginate($perPage)->withQueryString();
        $this->data->load("media");
    }

    private function getVideo(string $slug)
    {
        $this->video = cache()->remember(FrontConst::CACHE['video_details'] . $slug, FrontConst::getCacheExpire(), function () use ($slug) {
            $videos = Video::active()->WhereSlug($slug)->firstOrFail();
            return $videos->load(['media', 'category', 'user', 'metatags']);
        });
    }

    private function getReletedVideos()
    {
        $this->data = cache()->remember(FrontConst::CACHE['video_releted'] . $this->video->getKey(), FrontConst::getCacheExpire(), function () {
            $videos = Video::active()->whereKeyNot($this->video->getKey())->reletedvideos($this->getVideoTags())->orderByDesc('published_at')->orderByDesc("today_views")->limit(setting(FrontConst::LIMIT_VIDEO_RELETED, FrontConst::LIMIT_DEFAULT_VIDEO_RELETED))->get(FrontConst::VIDEO_ITEMS_COLUMNS);
            return $videos->load('media');
        });
    }


    private function getPlaylists()
    {

        $playlists = cache()->remember(FrontConst::CACHE['video_releted_playlist'] . $this->video->getKey(), FrontConst::getCacheExpire(), function () {
            $playlists = Playlist::whereDate("created_at", "<=", Carbon::parse($this->video->published_at))->withCount('videos')->active()->limit(setting(FrontConst::LIMIT_RELETED_PLAYLIST_LIMIT, FrontConst::LIMIT_DEFAULT_RELETED_PLAYLIST))->get(FrontConst::PLAYLIST_ITEMS_COLUMNS);
            return $playlists->load([FrontConst::MODEL_MEDIA]);
        });
        return $playlists;
    }

    private function getVideoTags()
    {
        return cache()->remember(FrontConst::CACHE['video_tags'] . $this->video->getKey(), FrontConst::getCacheExpire(), function () {
            $titles = Str::of($this->video->title)->remove([',', '-', '|', '/'])->explode(' ');
            $tags = $this->video->tags_list;
            $totalwords = count($titles);
            if ($totalwords > 2) {
                $i = 0;
                $j = 0;
                while ($j <= $totalwords) {
                    $tags[] = $titles[$i] . ' ' . $titles[$i + 1];
                    $j = $j + 2;
                    $i++;
                }
            }
            return $tags;
        });
    }

    private function setMetas()
    {
        $title = trans('meta_tags.videos.title');
        $desc = trans('meta_tags.videos.description');

        Meta::prependTitle($title);
        Meta::setDescription($desc);

        if ($keywords =  trans('meta_tags.videos.keywords')) {
            Meta::setKeywords($keywords);
        }

        $this->meta_og->setType('article')
            ->setTitle($title)
            ->setDescription($desc);

        $this->meta_og->addImage(asset(FrontConst::META_OG_IMAGE));

        Meta::registerPackage($this->meta_og);
    }



    private function setTopMetas()
    {

        $title = trans('meta_tags.topvideos.title');
        $desc = trans('meta_tags.topvideos.description');

        Meta::prependTitle($title);
        Meta::setDescription($desc);

        if ($keywords =  trans('meta_tags.topvideos.keywords')) {
            Meta::setKeywords($keywords);
        }

        $this->meta_og->setType('article')
            ->setTitle($title)
            ->setDescription($desc);

        $this->meta_og->addImage(asset(FrontConst::META_OG_IMAGE));

        Meta::registerPackage($this->meta_og);
    }

    private function setVideoMetas()
    {
        //   $username = $this->video->user ? trans('meta_tags.video.user', ['username' => $this->video->user->user_name]) : null;
        $categoryname = $this->video->category ? trans('meta_tags.video.category', ['categoryname' => $this->video->category->name]) : null;

        $title = $this->video->meta_title ?: trans(
            'meta_tags.video.title',
            ['title' => $this->video->title]
        );

        $desc = $this->video->meta_description ?: trans(
            'meta_tags.video.description',
            [
                'title' => $this->video->title,
                //  'username' => $username,
                'category' => $categoryname,
                'published' => $this->video->published,
            ]
        );
        Meta::prependTitle($title);

        Meta::setDescription($desc);
        if ($keywords = $this->video->meta_keywords ?: trans(
            'meta_tags.video.keywords',
            [
                'title' => $this->video->title,
                // 'username' => $username,
                'category' => $categoryname,
                'published' => $this->video->published,
            ]
        )) {
            Meta::setKeywords($keywords);
        }


        $this->meta_og
            ->setType('article')
            ->setTitle($title)
            ->setDescription($desc);




        if ($this->contentUri) {
            $this->schema = [
                "@context" => "https://schema.org",
                "@type" => "VideoObject",
                "name" => $this->video->title,
                "description" => $desc,
                "thumbnailUrl" => optional($this->video->getFirstMedia(MyConstant::THUMB_COLLECTION))->getResponsiveImageUrls(MyConstant::THUMB) ?: asset('images/default.jpg'),
                "uploadDate" => Carbon::createFromFormat("Y-m-d H:i", $this->video->published_at)->toIso8601String(),
                "contentUrl" => $this->contentUri,
                "interactionStatistic" => [
                    "@type" => "InteractionCounter",
                    "interactionType" => ["@type" =>  "WatchAction"],
                    "userInteractionCount" => $this->video->views
                ],
            ];
        }

        Meta::registerPackage($this->meta_og);
    }

    private function setTagMetas()
    {

        $title = trans('meta_tags.tagvideos.title', ['title' => $this->query]);
        $desc = trans('meta_tags.tagvideos.description', ['title' => $this->query]);

        Meta::prependTitle($title);
        Meta::setDescription($desc);

        if ($keywords = trans('meta_tags.tagvideos.keywords', ['title' => $this->query])) {
            Meta::setKeywords($keywords);
        }
        $this->meta_og->setType('article')
            ->setTitle($title)
            ->setDescription($desc);

        $this->meta_og->addImage(asset(FrontConst::META_OG_IMAGE));

        Meta::registerPackage($this->meta_og);
    }

    private function setSearchMetas()
    {
        $title = trans('meta_tags.search.title', ['title' => $this->query]);
        $desc = trans('meta_tags.search.description', ['title' => $this->query]);


        Meta::prependTitle($title);
        Meta::setDescription($desc);

        if ($keywords = trans('meta_tags.search.keywords', ['title' => $this->query])) {
            Meta::setKeywords($keywords);
        }
        $this->meta_og->setType('article')
            ->setTitle($title)
            ->setDescription($desc);

        $this->meta_og->addImage(asset(FrontConst::META_OG_IMAGE));
        Meta::registerPackage($this->meta_og);
    }
}
