<?php

namespace App\Jobs;

use App\Models\Video;
use App\Services\UploadService;
use App\Utils\PendingVideo;
use Exception;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Storage;
use Illuminate\Support\Str;
use MyConstant;
use ProtoneMedia\LaravelFFMpeg\Support\FFMpeg;
use ProtoneMedia\LaravelFFMpeg\MediaOpener;
use FFMpeg\Format\Video\X264;
use FFMpeg\Filters\Video\VideoFilters;
use FFMpeg\Coordinate\Dimension;
use Illuminate\Contracts\Queue\ShouldBeUnique;
use Illuminate\Support\Collection;
use ProtoneMedia\LaravelFFMpeg\Filters\WatermarkFactory;
use Spatie\Image\Manipulations;
use Throwable;

class VideoUploadingJob implements ShouldQueue, ShouldBeUnique
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    protected PendingVideo $pendingVideo;

    protected Video $videoModel;

    protected UploadService $fileHandler;

    protected MediaOpener $mediaOpener;

    protected $videoHeight;

    protected $videoWidth;

    protected $videoRatio;

    protected bool $isLandscape = true;

    protected ?Collection $resolutions = null;

    /**
     * Undocumented function
     *
     * @param Video $video
     * @param string $videoPath
     * @param string $coverPath
     * @param boolean $conversion
     */
    public function __construct(PendingVideo $pendingVideo)
    {
        $this->pendingVideo = $pendingVideo;
        $this->videoModel = $pendingVideo->video;
        $this->fileHandler = new UploadService($this->videoModel);
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        $this->videoModel->is_uploaded = 3;
        $this->videoModel->save();

        if ($this->instanceMediaOpener()) {
            $this->updateDuration();
            $this->updateThumbnail();

            if ($this->pendingVideo->conversion) {
                $this->uploadConversion();
                $this->videoModel->is_uploaded = 1;
                $this->videoModel->active = $this->pendingVideo->isActive;
                $this->videoModel->save();

                Storage::disk("local")->delete($this->pendingVideo->videoPath);
            } else {
                $this->uploadVideo(true);
            }
        } else {
            $this->uploadVideo();
        }
    }


    /**
     * Video upload when conversion false or not converteted
     *
     * @param boolean $isanalysis
     * @return boolean
     */
    public function uploadVideo(bool $isAnalysis = false): bool
    {

        if ($this->isVideo($this->pendingVideo->videoPath) && Storage::disk($this->pendingVideo->videoDisk)->exists($this->pendingVideo->videoPath)) {

            $this->fileHandler->setFrom(UploadService::FILE_ADD_FROM_DISK, $this->pendingVideo->videoDisk)
                ->setFile($this->pendingVideo->videoPath);
        } else  if ($this->pendingVideo->isVideoUrl && $this->pendingVideo->videoPath) {

            $this->fileHandler->setFrom(UploadService::FILE_FROM_URL)
                ->setUrl($this->pendingVideo->videoPath, MyConstant::ALLOW_URL_VIDEO_UPLOAD);
        } else {
            info("VideoUploadingJob: video not found or not url" . $this->pendingVideo->videoPath);
            return false;
        }

        $uploadMedia = $this->fileHandler->setTitle($this->videoModel->title)->setCollection(MyConstant::VIDEO_COLLECTION);

        if ($isAnalysis) {
            $this->initResolutions();
            $uploadMedia->setCustomProperties(['width' => $this->videoWidth, 'height' => $this->videoHeight]);
        }

        $uploadMedia->save();

        info("VideoUploadingJob: Uploading a orginal Video Successfuly");

        $this->videoModel->is_uploaded = 1;
        $this->videoModel->active = $this->pendingVideo->isActive;
        $this->videoModel->save();

        return true;
    }

    /**
     * Video conversion and upload
     *
     * @return void
     */
    public function uploadConversion()
    {
        $this->initResolutions();

        if ($this->resolutions->isEmpty()) {

            info("VideoUploadingJob: No resolutions found to converting");
            $this->uploadVideo();
            return;
        }

        try {
            if (setting(MyConstant::VIDEO_WATERMARK) == "ON") {
                $this->addVideoWaterMark($this->mediaOpener, $this->videoWidth);
            }

            foreach ($this->resolutions as $kiloBitrate => $vH) {
                if ($this->isLandscape) {
                    $vW = $this->videoRatio->calculateWidth($vH, 2);
                } else {
                    $vW = $vH;
                    $vH = $this->videoRatio->calculateHeight($vH, 2);
                }
                $pathToSave = Str::random() . '.' . MyConstant::MP4_VIDEO_FORMAT;
                $this->mediaOpener->export()
                    ->inFormat((new X264)->setKiloBitrate($kiloBitrate))
                    ->addFilter(function (VideoFilters $filters) use ($vW, $vH) {
                        $filters->resize(new Dimension($vW, $vH));
                    })
                    ->toDisk("local")
                    ->save($pathToSave);
                $this->attachedConversion("local", $pathToSave, $vH, $vW);
            }
        } catch (Exception $e) {
            info("VideoUploadingJob: Error to converting " . $e->getMessage());
            $this->uploadVideo();
        }
    }


    /**
     * Analysis Video
     *
     * @return void
     */

    public function instanceMediaOpener()
    {

        try {
            if ($this->pendingVideo->isVideoUrl) {

                $this->mediaOpener = FFMpeg::openUrl($this->pendingVideo->videoPath);
            } else {
                $this->mediaOpener = FFMpeg::fromDisk($this->pendingVideo->videoDisk)->open($this->pendingVideo->videoPath);
            }
        } catch (Exception $e) {
            info("VideoUploadingJob: " . $e->getMessage());
            return false;
        }

        return true;
    }

    /**
     * init initResolutions;
     *
     * @return void
     */
    public function initResolutions(): void
    {
        $videoInfo = $this->mediaOpener->getVideoStream()->getDimensions();

        $this->videoHeight = $videoInfo->getHeight();

        $this->videoWidth = $videoInfo->getWidth();

        $this->videoRatio =  $videoInfo->getRatio();

        if ($this->videoWidth > $this->videoHeight) {
            $myHeight =  $this->videoHeight;
            $this->isLandscape = true;
            info("LandscapeVideo");
        } else {
            $myHeight = $this->videoWidth;
            $this->isLandscape = false;
            info("not Landscape");
        }

        $this->resolutions =  collect(MyConstant::VIDEO_RESOLUTION)->reject(function ($h) use ($myHeight) {
            return  $h > $myHeight;
        })
            ->sortDesc();
    }

    public function updateDuration()
    {
        $videoDuration = $this->mediaOpener->getDurationInSeconds();
        $duration = gmdate("H:i:s", $videoDuration);
        $this->videoModel->duration = $duration;
        $this->videoModel->save();
    }

    public function updateThumbnail()
    {
        if (!$this->videoModel->getFirstMedia(MyConstant::THUMB_COLLECTION)) {
            $filepath = 'tmp/' . Str::random() . '.jpg';
            $this->mediaOpener->clone()->getFrameFromSeconds(MyConstant::THUMB_EXTRACT_AT)
                ->export()
                ->toDisk('local')
                ->save($filepath);

            $this->fileHandler->setFrom(UploadService::FILE_ADD_FROM_DISK, 'local')
                ->setFile($filepath)
                ->setTitle($this->videoModel->title)
                ->setCollection(MyConstant::THUMB_COLLECTION)
                ->save();
        }
    }


    /**
     * upload conversion
     *
     * @param string $disk
     * @param string $path
     * @param integer $size
     * @return void
     */
    protected function attachedConversion(string $disk, string $path, int $size, int $width): void
    {
        $this->fileHandler->setFrom(UploadService::FILE_ADD_FROM_DISK, $disk)
            ->setFile($path)
            ->setTitle($this->videoModel->title)
            ->setCollection(MyConstant::VIDEO_COLLECTION)
            ->setCustomProperties(['width' => $width, 'height' => $size])
            ->save();
    }


    protected function addVideoWaterMark($videoOpener, $vW)
    {
        $videoOpener->addWatermark(function (WatermarkFactory $watermark) use ($vW) {
            $offset = setting(MyConstant::VIDEO_WATERMARK_OFFSET, 0);
            $width = ceil(($vW / 100) * setting(MyConstant::VIDEO_WATERMARK_WIDTH, 30));
            $watermark = $watermark->fromDisk('local')
                ->open('images/video_watermark.png');
            switch (setting(MyConstant::VIDEO_WATERMARK_POSITION, Manipulations::POSITION_BOTTOM_RIGHT)) {
                case Manipulations::POSITION_TOP_LEFT:
                    $watermark->horizontalAlignment(WatermarkFactory::LEFT, $offset)
                        ->verticalAlignment(WatermarkFactory::TOP, $offset);
                    break;
                case Manipulations::POSITION_TOP:
                    $watermark->horizontalAlignment(WatermarkFactory::CENTER)
                        ->verticalAlignment(WatermarkFactory::TOP, $offset);
                    break;
                case Manipulations::POSITION_TOP_RIGHT:
                    $watermark->horizontalAlignment(WatermarkFactory::RIGHT, $offset)
                        ->verticalAlignment(WatermarkFactory::TOP, $offset);
                    break;
                case Manipulations::POSITION_LEFT:
                    $watermark->horizontalAlignment(WatermarkFactory::LEFT, $offset)
                        ->verticalAlignment(WatermarkFactory::CENTER);
                    break;
                case Manipulations::POSITION_CENTER:
                    $watermark->horizontalAlignment(WatermarkFactory::CENTER)
                        ->verticalAlignment(WatermarkFactory::CENTER);
                    break;
                case Manipulations::POSITION_RIGHT:
                    $watermark->horizontalAlignment(WatermarkFactory::RIGHT, $offset)
                        ->verticalAlignment(WatermarkFactory::CENTER);
                    break;
                case Manipulations::POSITION_BOTTOM_LEFT:
                    $watermark->horizontalAlignment(WatermarkFactory::LEFT, $offset)
                        ->verticalAlignment(WatermarkFactory::BOTTOM, $offset);
                    break;
                case Manipulations::POSITION_BOTTOM:
                    $watermark->horizontalAlignment(WatermarkFactory::CENTER)
                        ->verticalAlignment(WatermarkFactory::BOTTOM, $offset);
                    break;
                case Manipulations::POSITION_BOTTOM_RIGHT:
                    $watermark->horizontalAlignment(WatermarkFactory::RIGHT, $offset)
                        ->verticalAlignment(WatermarkFactory::BOTTOM, $offset);
                    break;
                default:
                    $watermark->horizontalAlignment(WatermarkFactory::LEFT, $offset)
                        ->verticalAlignment(WatermarkFactory::TOP, $offset);
                    break;
            }
            $watermark->width($width)
                ->optimize();
        });
    }




    public function uniqueId()
    {
        return $this->videoModel->getKey();
    }

    protected function isVideo(string $filename): bool
    {
        $extension = pathinfo($filename, PATHINFO_EXTENSION);
        return in_array($extension,  ['mp4', 'mov', 'wmv', 'mkv', 'webm']);
    }

    /**
     * Handle a job failure.
     *
     * @param  \Throwable  $exception
     * @return void
     */
    public function failed(Throwable $exception)
    {
        $this->videoModel->is_uploaded = 0;
        $this->videoModel->active = false;
        $this->videoModel->save();
        info("VideoUploadingJob: " . $exception->getMessage());
    }
}
