<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use YouTube\YouTubeDownloader;
use YouTube\Exception\YouTubeException;
use Curl\Client;
use CurlDownloader\CurlDownloader;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Storage;
use Illuminate\Http\File;
use App\Services\UploadService;
use App\Utils\MyConstant;
use App\Utils\PendingVideo;
use Exception;
use ProtoneMedia\LaravelFFMpeg\Support\FFMpeg;
use FFMpeg\Format\Video\X264;
use ProtoneMedia\LaravelFFMpeg\Exporters\EncodingException;
use ProtoneMedia\LaravelFFMpeg\Filesystem\Media;
use Log;
use Str;

class YoutubeVideoCopyToServer implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;


    protected bool $conversion;

    protected ?string $videoUrl = null;

    protected bool $status;

    protected ?Model $model = null;

    protected string $sourceQuality = "medimum";

    protected ?Collection $ytLinks = null;

    protected bool $isFileCopied = false;

    protected ?string $fileToSave = null;

    protected ?string $ytVideoPath = null;

    protected ?string $ytAudioPath = null;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct(
        Model $model,
        bool $status,
        string $sourceQuality = "medium",
        bool $conversion = false
    ) {
        $this->model = $model;
        $this->status = $status;
        $this->sourceQuality = $sourceQuality;
        $this->conversion = $conversion;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        //

        if (!$this->model) {
            return false;
        }

        if (!$this->model->youtube_id) {
            return false;
        }

        $youtube = new YouTubeDownloader;
        try {
            $ytRequest = $youtube->getDownloadLinks($this->model->youtube_id);
            if ($ytLinks = $ytRequest->getCombinedFormats()) {
                $this->ytLinks = collect($ytLinks);
            } else 
            if ($ytVideoAndAudio = $ytRequest->getSplitFormats($this->sourceQuality)) {
                $this->mergeVideoAndAudio($ytVideoAndAudio->video->url, $ytVideoAndAudio->audio->url);
            } else {
                info("YoutubeVideoCopyToServer: " . $this->model->youtube_id . " Video Strem links not found!");
            }
        } catch (YouTubeException $e) {
            info("YoutubeVideoCopyToServer: " . $e->getMessage() . " Video not found!");
        }

        if ($this->ytLinks && $this->ytLinks->isNotEmpty()) {
            if ($this->sourceQuality == "low") {
                $this->videoUrl = $this->ytLinks->first()->url;
            } else if ($this->sourceQuality == "high") {
                $this->videoUrl = $this->ytLinks->last()->url;
            } else {
                $mediumIndex = $this->ytLinks->count() / 2;
                $mediumIndex = floor($mediumIndex);
                $mediumIndex = intval($mediumIndex);
                $this->videoUrl = $this->ytLinks->get($mediumIndex)->url;
            }
        }

        if ($this->videoUrl) {
            $browser = new Client();
            $downloader = new CurlDownloader($browser);
            $response = $downloader->download($this->videoUrl, function ($filename) {
                $disk = Storage::disk('local');
                if (!$disk->exists("tmp")) {
                    $disk->makeDirectory("tmp");
                }
                $extension = pathinfo($filename, PATHINFO_EXTENSION);
                $this->fileToSave = 'tmp/' . $this->model->id . '.' . $extension;
                return $disk->path($this->fileToSave);
            });
            if ($response->status == 200) {
                $this->isFileCopied = true;
                info("YoutubeVideoCopyToController: " . number_format($response->info->size_download) . ' bytes downloaded in ' . $response->info->total_time . ' seconds');
            } else {
                info("YoutubeVideoCopyToController: " . $this->model->youtube_id . " Video not yet copyied to server");
            }
        }

        if ($this->isFileCopied && $this->fileToSave) {
            $pendingVideo = new PendingVideo;
            $pendingVideo->video = $this->model;
            $pendingVideo->videoPath = $this->fileToSave;
            $pendingVideo->conversion  = $this->conversion;
            $pendingVideo->isActive = $this->status;
            VideoUploadingJob::dispatchSync($pendingVideo);
        }
    }

    protected function mergeVideoAndAudio($videoUrl, $audioUrl)
    {

        if (!$videoUrl || !$audioUrl) {
            return false;
        }

        $browser = new Client();
        $downloader = new CurlDownloader($browser);

        if (!Storage::disk('local')->exists("tmp")) {
            Storage::disk('local')->makeDirectory("tmp");
        }

        $vidResponse = $downloader->download($videoUrl, function ($filename) {
            $extension = pathinfo($filename, PATHINFO_EXTENSION);
            $this->ytVideoPath = 'tmp/' . Str::random() . '.' . $extension;
            return Storage::disk('local')->path($this->ytVideoPath);
        });

        if (!$vidResponse->status == 200) {
            return false;
        }

        $AudioResponse = $downloader->download($audioUrl, function ($filename) {
            $extension = pathinfo($filename, PATHINFO_EXTENSION);
            $this->ytAudioPath = 'tmp/audio_' . $this->model->id . '.' . $extension;
            return Storage::disk('local')->path($this->ytAudioPath);
        });

        if (!$AudioResponse->status == 200) {
            return false;
        }

        if (!$this->ytVideoPath || !$this->ytAudioPath) {
            return false;
        }

        try {

            $ytVidInfo = FFMpeg::fromDisk('local')
                ->open($this->ytVideoPath)->getVideoStream();

            $ytAudioInfo = FFMpeg::fromDisk('local')
                ->open($this->ytAudioPath)->getAudioStream();


            if (!$ytVidInfo || !$ytAudioInfo) {
                return false;
            }

            $minVideoKilobyte = 350;
            $minAudioKilobyte = 96;

            $ytVideobyte = intval($ytVidInfo->get('bit_rate'));
            $ytAudiobyte = intval($ytAudioInfo->get('bit_rate'));

            $ytVideoKilobyte = intval($ytVideobyte / 1000);
            $ytAudioKilobyte = intval($ytAudiobyte / 1000);

            while ($ytVideoKilobyte < $minVideoKilobyte) {
                $ytVideoKilobyte++;
            }

            while ($ytAudioKilobyte < $minAudioKilobyte) {
                $ytAudioKilobyte++;
            }

            $ytVideoToSave = 'tmp/' . Str::random() . '.' . MyConstant::MP4_VIDEO_FORMAT;

            FFMpeg::fromDisk('local')
                ->open([$this->ytVideoPath, $this->ytAudioPath])
                ->export()
                ->addFormatOutputMapping((new X264)->setKiloBitrate($ytVideoKilobyte)->setAudioKiloBitrate($ytAudioKilobyte), Media::make('local', $ytVideoToSave), ['0:v', '1:a'])
                ->save();

            $this->fileToSave = $ytVideoToSave;
            $this->isFileCopied = true;

            Storage::disk('local')->delete([$this->ytVideoPath, $this->ytAudioPath]);
        } catch (Exception $e) {
            Log::alert($e->getMessage());
        }
    }
}
