<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Spatie\MediaLibrary\HasMedia;
use Spatie\MediaLibrary\InteractsWithMedia;
use Spatie\MediaLibrary\MediaCollections\Models\Media;
use Cviebrock\EloquentSluggable\Sluggable;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use App\Utils\MyConstant;
use Spatie\Image\Manipulations;
use App\Utils\FrontConst;
use Cviebrock\EloquentSluggable\SluggableScopeHelpers;
use Illuminate\Support\Arr;

class Category extends BaseModel implements HasMedia
{
    use HasFactory;
    use InteractsWithMedia;
    use Sluggable;
    use SluggableScopeHelpers;


    protected $fillable = [
        'name',
        'ord',
        'slug'
    ];


    /**
     * The attributes that should be cast.
     *
     * @var array
     */
    protected $casts = [
        'active' => 'boolean',
    ];


    public function videos()
    {
        return $this->hasMany(Video::class);
    }

    public function scopeParents(Builder $query)
    {

        return $query->whereNull('category_id');
    }

    public function childrens()
    {
        return $this->hasMany(Category::class, 'category_id');
    }

    public function childrensVideos()
    {
        return $this->hasManyThrough(Video::class, Category::class, 'category_id');
    }

    public function scopeActive(Builder $query)
    {
        return $query->where("active", 1);
    }



    /**
     * custom sort
     *
     * @param Builder $query
     * @param string $sort
     */
    public function scopeCustomSort(Builder $query, string $sort)
    {
        switch ($sort) {
            case FrontConst::SORT_LATEST:
                $query->latest();
                break;
            case FrontConst::SORT_RELEVANCE:
                $query->orderBy("ord");
                break;
            case FrontConst::SORT_AZ:
                $query->orderBy('name');
                break;
            case FrontConst::SORT_ZA:
                $query->orderByDesc('name');
                break;
            case FrontConst::SORT_OLDEST:
                $query->oldest();
                break;
            default:
                $query->latest();
                break;
        }
        return $query;
    }
    /**
     * get Category Filters
     *
     * @param array|null $parameters
     * @return array
     */
    public static function getFilters(?array $parameters = null): array
    {
        $filters = array();
        $filters[] = ['name' => __('Latest'), 'parameters' => Arr::add($parameters, 'sort', FrontConst::SORT_LATEST)];
        $filters[] = ['name' => __('Relevance'), 'parameters' => Arr::add($parameters, 'sort', FrontConst::SORT_RELEVANCE)];
        $filters[] = ['name' => __('A to Z'), 'parameters' => Arr::add($parameters, 'sort', FrontConst::SORT_AZ)];
        $filters[] = ['name' => __('Z to A'), 'parameters' => Arr::add($parameters, 'sort', FrontConst::SORT_ZA)];
        $filters[] = ['name' => __('Oldest'), 'parameters' => Arr::add($parameters, 'sort', FrontConst::SORT_OLDEST)];
        return $filters;
    }

    /**
     * set or update meta tags
     * 
     * @param array $attributes
     * @param array $values
     * @return \Illuminate\Database\Eloquent\Model
     */

    public function setMetaTags(array $attributes, array $values = []): Model
    {

        return $this->metatags()->updateOrCreate($attributes, $values);
    }

    public function sluggable(): array
    {
        return [
            'slug' => [
                'source' => 'name'
            ]
        ];
    }

    public function registerMediaConversions(Media $media = null): void
    {
        $conversion = $this->addMediaConversion(MyConstant::THUMB)
            ->fit(Manipulations::FIT_CROP, 1280, 720)
            ->background(MyConstant::IMAGEBG_FILL_COLOR);

        if (setting(MyConstant::IMAGE_WATERMARK) == "ON") {
            $conversion->watermark(storage_path('app/images/image_watermark.png'))
                ->watermarkPosition(setting(MyConstant::IMAGE_WATERMARK_POSITION, Manipulations::POSITION_TOP_RIGHT))
                ->watermarkWidth(setting(MyConstant::IMAGE_WATERMARK_WIDTH, 25), Manipulations::UNIT_PERCENT)
                ->watermarkPadding(setting(MyConstant::IMAGE_WATERMARK_PADDING, 2), null, Manipulations::UNIT_PERCENT);
        }
        $conversion->optimize()
            ->withResponsiveImages()
            ->performOnCollections(MyConstant::THUMB_COLLECTION);
    }
}
