<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Cviebrock\EloquentSluggable\Sluggable;
use Spatie\MediaLibrary\HasMedia;
use Spatie\MediaLibrary\InteractsWithMedia;
use Illuminate\Database\Eloquent\Builder;
use App\Utils\MyConstant;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use App\Models\PlaylistVideo;
use Spatie\MediaLibrary\MediaCollections\Models\Media;
use Spatie\Image\Manipulations;
use App\Utils\FrontConst;
use Cviebrock\EloquentSluggable\SluggableScopeHelpers;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Support\Arr;

class Playlist extends BaseModel implements HasMedia
{
    use HasFactory, InteractsWithMedia, Sluggable, SluggableScopeHelpers;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'name',
        'description',
    ];

    public function sluggable(): array
    {
        return [
            'slug' => [
                'source' => 'name'
            ]
        ];
    }

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'active' => 'boolean',
    ];


    /**
     * Registering Media conversion
     *
     * @param Media|null $media
     * @return void
     */
    public function registerMediaConversions(Media $media = null): void
    {
        $conversion = $this->addMediaConversion(MyConstant::THUMB)
            ->fit(Manipulations::FIT_CROP, 1280, 720)
            ->background(MyConstant::IMAGEBG_FILL_COLOR);

        if (setting(MyConstant::IMAGE_WATERMARK) == "ON") {
            $conversion->watermark(storage_path('app/images/image_watermark.png'))
                ->watermarkPosition(setting(MyConstant::IMAGE_WATERMARK_POSITION, Manipulations::POSITION_TOP_RIGHT))
                ->watermarkWidth(setting(MyConstant::IMAGE_WATERMARK_WIDTH, 25), Manipulations::UNIT_PERCENT)
                ->watermarkPadding(setting(MyConstant::IMAGE_WATERMARK_PADDING, 2), null, Manipulations::UNIT_PERCENT);
        }
        $conversion->optimize()
            ->withResponsiveImages()
            ->performOnCollections(MyConstant::THUMB_COLLECTION);
    }
    /**
     * get user of the playlist
     *
     * @return BelongsTo
     */
    public function user(): BelongsTo
    {
        return $this->belongsTo(User::class);
    }

    /**
     * get Playlist videos
     *
     * @return BelongsToMany
     */
    public function videos(): BelongsToMany
    {
        return $this->belongsToMany(video::class)->using(PlaylistVideo::class)->withTimestamps();
    }


    /**
     * get Playlist videos order by id asc
     *
     * @return BelongsToMany
     */
    public function videosByUpdated(): BelongsToMany
    {
        return $this->belongsToMany(video::class)->using(PlaylistVideo::class)->orderByPivot('updated_at');
    }

    /**
     * set or update meta tags
     *
     * @param array $attributes
     * @param array $values
     * @return Model
     */
    public function setMetaTags(array $attributes, array $values = []): Model
    {

        return $this->metatags()->updateOrCreate($attributes, $values);
    }


    public function scopeActive(Builder $query)
    {

        return $query->where('active', 1);
    }

    /**
     * custom sort
     *
     * @param Builder $query
     * @param string $sort
     */
    public function scopeCustomSort(Builder $query, string $sort)
    {
        switch ($sort) {
            case FrontConst::SORT_LATEST:
                $query->latest();
                break;
            case FrontConst::SORT_RELEVANCE:
                $query->orderBy("ord");
                break;
            case FrontConst::SORT_AZ:
                $query->orderBy('name');
                break;
            case FrontConst::SORT_ZA:
                $query->orderByDesc('name');
                break;
            case FrontConst::SORT_OLDEST:
                $query->oldest();
                break;
            default:
                $query->latest();
                break;
        }
        return $query;
    }

    /**
     * get Playlist filters
     *
     * @param array|null $parameters
     * @return array
     */
    public static function getFilters(?array $parameters = null): array
    {
        $filters = array();
        $filters[] = ['name' => __('Latest'), 'parameters' => Arr::add($parameters, 'sort', FrontConst::SORT_LATEST)];
        $filters[] = ['name' => __('A to Z'), 'parameters' => Arr::add($parameters, 'sort', FrontConst::SORT_AZ)];
        $filters[] = ['name' => __('Z to A'), 'parameters' => Arr::add($parameters, 'sort', FrontConst::SORT_ZA)];
        $filters[] = ['name' => __('Oldest'), 'parameters' => Arr::add($parameters, 'sort', FrontConst::SORT_OLDEST)];
        return $filters;
    }
}
