<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Cache;

/**
 * App\Models\Setting
 *
 * @property int $id
 * @property string $name
 * @property string|null $value
 * @property \Illuminate\Support\Carbon|null $created_at
 * @property \Illuminate\Support\Carbon|null $updated_at
 * @property-read mixed $values
 * @method static \Illuminate\Database\Eloquent\Builder|Setting newModelQuery()
 * @method static \Illuminate\Database\Eloquent\Builder|Setting newQuery()
 * @method static \Illuminate\Database\Eloquent\Builder|Setting query()
 * @mixin \Eloquent
 */
class Setting extends Model
{
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */

    protected $fillable = ['name', 'value'];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */

    protected $casts = [

    ];

   /**
     * The event map for the model.
     *
     * @var array
     */
    protected $dispatchesEvents = [
    ];


    /**
     * Get all settings with cache support.
     *
     * @return \Illuminate\Database\Eloquent\Collection
     */

    public static function getAllCached()
    {
        return Cache::rememberForever('settings.all', function () {
            return self::all()->mapWithKeys(function ($setting) {
                return [$setting->name => $setting->values];
            });
        });
    }

    /**
     * Determine if the given setting name exists.
     *
     * @param string $name
     * @return bool
     */

    public static function has($name)
    {
        return static::where('name', $name)->exists();
    }

    /**
     * Get setting for the given name.
     *
     * @param string $name
     * @param mixed $default
     * @return string|array
     */
    public static function get($name, $default = null)
    {
        return static::where('name', $name)->first()->values ?? $default;
    }

    /**
     * Set the given setting.
     *
     * @param string $name
     * @param mixed $value
     * @return void
     */
    public static function set($name, $value)
    {
        static::updateOrCreate(['name' => $name], ['value' => $value]);
    }

    /**
     * Set the given settings.
     *
     * @param array $settings
     * @return void
     */

    public static function setMany($settings)
    {
        foreach ($settings as $name => $value) {
            self::set($name, $value);
        }
    }

    /**
     * Get the value of the setting.
     *
     * @return mixed
     */
    public function getValuesAttribute()
    {
        return unserialize($this->value);
    }

    /**
     * Set the value of the setting.
     *
     * @param mixed $value
     * @return mixed
     */
    public function setValueAttribute($value)
    {
        $this->attributes['value'] = serialize($value);
    }
}
