<?php

namespace App\Models;

use App\Utils\FrontConst;
use App\Utils\MyConstant;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Spatie\MediaLibrary\HasMedia;
use Spatie\MediaLibrary\InteractsWithMedia;
use Cviebrock\EloquentSluggable\Sluggable;
use Cviebrock\EloquentSluggable\SluggableScopeHelpers;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Spatie\MediaLibrary\MediaCollections\Models\Media;
use Spatie\Image\Manipulations;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;
use Str;
use App\Traits\VideoUrlHelpers;
use Carbon\Carbon;

class Video extends BaseModel implements HasMedia
{
    use HasFactory, InteractsWithMedia, Sluggable, SluggableScopeHelpers, VideoUrlHelpers;


    /**
     * The attributes that are mass assignable.
     * 
     *
     * @var array
     */

    protected $fillable = ['title', 'category_id', 'youtube_id', 'tags', 'description', 'published_at'];



    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'likes' => 'integer',
        'unlikes' => 'integer',
        'today_views' => 'integer',
        'views' => 'integer',
        'is_download' => 'boolean',
        'featured' => 'boolean',
        'active' => 'boolean',
        'duration' => 'string',
        'is_uploaded' => 'integer',
        'published_at' => "timestamp"
    ];

    /**
     * get User Name of the video
     *
     * @return string
     */
    public function getUserNameAttribute(): string
    {
        return $this->user->name;
    }

    /**
     * get category name of the video
     *
     * @return string
     */
    public function getCategoryNameAttribute(): string
    {
        return $this->category ? $this->category->name : '';
    }


    public function getPublishedAttribute()
    {
        return Carbon::parse($this->attributes['published_at'])->format("M d, Y");
    }

    /**
     * get category name of the video
     *
     * @return string
     */
    public function getPublishedAtAttribute(): string
    {
        return Carbon::parse($this->attributes['published_at'])->format("Y-m-d H:i");
    }

    /**
     * get Duration in H:I:S
     *
     * @return string
     */
    public function getDurationAttribute(): string
    {
        if ($duration = $this->attributes['duration']) {
            $duration = Str::replaceFirst("00:", "", $duration);
            $duration = Str::replaceFirst("0", "", $duration);
            return $duration;
        }
        return "";
    }

    public function getVideoDurationAttribute(): string
    {
        return $this->attributes['duration'];
    }

    public function getPublishedAtStatusAttribute(): string
    {
        $datetime =  Carbon::parse($this->attributes['published_at']);
        if ($datetime->isPast()) {
            return '<span class="text-success">' . $datetime->format("Y-m-d H:i") . '</span>';
        }
        return '<span class="text-danger">' . $datetime->format("Y-m-d H:i") . '</span>';
    }


    public function getActiveStatusAttribute(): string
    {
        return ($this->attributes['active']) ? '<span class="badge badge-success">Public</span>' : '<span class="badge badge-warning">Private</span>';
    }


    public function getUploadStatusAttribute(): string
    {
        switch ($this->attributes['is_uploaded']) {
            case  1:
                $badge = '<span class="small">Successfuly</span>';
                break;

            case  2:
                $badge = '<span class="small">Pending</span>';
                break;

            case  3:
                $badge = '<span class="small">Processing</span>';
                break;

            default:
                $badge = '<span class="small">Failed</span>';
                break;
        }
        return $badge;
    }


    /**
     * getTagsList
     *
     * @return Collection
     */
    public function getTagsListAttribute(): array
    {
        return Str::of($this->tags)->explode(',')->filter()->all();
    }

    /**
     * set metatags of the videos
     *
     * @param array $attributes
     * @param array $values
     * @return Model
     */
    public function setMetaTags(array $attributes, array $values = []): Model
    {

        return $this->metatags()->updateOrCreate($attributes, $values);
    }



    /**
     * Activated
     *
     * @param Builder $query
     * @return Builder
     */
    public function scopeActive(Builder $query): Builder
    {

        return $query->where('active', 1)->where("published_at", '<=', Carbon::now());
    }

    /**
     * Trending
     *
     * @param Builder $query
     * @return Builder
     */
    public function scopeTrending(Builder $query): Builder
    {
        $day =  setting(FrontConst::LIMIT_HOME_TREND_DAY, 365) ?: 365;
        return $query->whereBetween('published_at', [now()->subDays($day), now()])->orderByDesc("likes")->orderBy("unlikes", "asc")->orderByDesc("today_views");
    }

    /**
     * Featured
     *
     * @param Builder $query
     * @return Builder
     */
    public function scopeFeatured(Builder $query): Builder
    {
        return $query->where('featured', true);
    }

    public function scopeReletedVideos(Builder $query, ?array $tags = null): Builder
    {
        return $query->where(function ($query) use ($tags) {
            $query->featured()->when($tags, function ($query, $tags) {
                foreach ($tags as $value) {
                    $query->orWhere('tags', 'LIKE', '%' . $value . '%');
                    $query->orWhere('title', 'LIKE', '%' . $value . '%');
                }
            });
        });
    }

    /**
     * Top Today
     *
     * @param Builder $query
     * @return Builder
     */
    public function scopeToptoday(Builder $query): Builder
    {
        return $query->orderByDesc("today_views");
    }

    /**
     * Sorting
     *
     * @param Builder $query
     * @param string $sort
     * @param boolean|null $qualify
     * @return Builder
     */
    public function scopeCustomSort(Builder $query, string $sort): Builder
    {
        switch ($sort) {
            case FrontConst::SORT_LATEST:
                $query->latest($this->qualifyColumn('published_at'));
                break;
            case FrontConst::SORT_RELEVANCE:
                $query->orderByDesc("today_views");
                break;
            case FrontConst::SORT_TRENDING:
                $query->trending();
                break;
            case FrontConst::SORT_AZ:
                $query->orderBy('title');
                break;
            case FrontConst::SORT_ZA:
                $query->orderByDesc('title');
                break;
            case FrontConst::SORT_OLDEST:
                $query->oldest($this->qualifyColumn('published_at'));
                break;
            default:
                $query->latest($query->qualifyColumn('published_at'));
                break;
        }

        return $query;
    }


    /**
     * get Video Filters
     *
     * @param array|null $parameters
     * @return array
     */
    public static function getFilters(?array $parameters = []): array
    {
        $filters = array();
        $filters[] = ['name' => __('Latest'), 'parameters' => Arr::add($parameters, 'sort', FrontConst::SORT_LATEST)];
        $filters[] = ['name' => __('Trending'), 'parameters' => Arr::add($parameters, 'sort', FrontConst::SORT_TRENDING)];
        $filters[] = ['name' => __('Relevance'), 'parameters' => Arr::add($parameters, 'sort', FrontConst::SORT_RELEVANCE)];
        $filters[] = ['name' => __('A to Z'), 'parameters' => Arr::add($parameters, 'sort', FrontConst::SORT_AZ)];
        $filters[] = ['name' => __('Z to A'), 'parameters' => Arr::add($parameters, 'sort', FrontConst::SORT_ZA)];
        $filters[] = ['name' => __('Oldest'), 'parameters' => Arr::add($parameters, 'sort', FrontConst::SORT_OLDEST)];
        return $filters;
    }


    public function sluggable(): array
    {
        return [
            'slug' => [
                'source' => 'title'
            ]
        ];
    }

    /**
     * Thumb Conversions
     *
     * @param Media|null $media
     * @return void
     */
    public function registerMediaConversions(Media $media = null): void
    {
        $conversion = $this->addMediaConversion(MyConstant::THUMB)
            ->fit(Manipulations::FIT_CROP, 1280, 720)
            ->background(MyConstant::IMAGEBG_FILL_COLOR);

        if (setting(MyConstant::IMAGE_WATERMARK) == "ON") {
            $conversion->watermark(storage_path('app/images/image_watermark.png'))
                ->watermarkPosition(setting(MyConstant::IMAGE_WATERMARK_POSITION, Manipulations::POSITION_TOP_RIGHT))
                ->watermarkWidth(setting(MyConstant::IMAGE_WATERMARK_WIDTH, 25), Manipulations::UNIT_PERCENT)
                ->watermarkPadding(setting(MyConstant::IMAGE_WATERMARK_PADDING, 2), null, Manipulations::UNIT_PERCENT);
        }
        $conversion->optimize()
            ->withResponsiveImages()
            ->performOnCollections(MyConstant::THUMB_COLLECTION);
    }


    /**
     * get videos Playlist
     *
     * @return BelongsToMany
     */
    public function playlist(): BelongsToMany
    {
        return $this->belongsToMany(Playlist::class);
    }

    /**
     * get User of the video
     *
     * @return BelongsTo
     */
    public function user(): BelongsTo
    {
        return $this->belongsTo(User::class);
    }

    /**
     * get Category of the video
     *
     * @return BelongsTo
     */
    public function category(): BelongsTo
    {
        return $this->belongsTo(Category::class);
    }

    /**
     * Video Views
     *
     * @return void
     */
    public function videoviews()
    {
        return $this->hasMany(VideoView::class);
    }
}
