<?php

namespace App\Services;

use App\Exceptions\ImageServiceException;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Str;

class UploadService
{
    const FILE_COLLECTION = 'files';
    const FILE_FROM_INPUT = "FILE";
    const FILE_FROM_URL = "URL";
    const FILE_FROM_BASE64 = "BASE64";
    const FILE_ADD_FROM_FILE = 'LOCAL';
    const FILE_ADD_FROM_DISK = 'DISK';

    private Model $model;

    private string $File;

    private string $Url;

    private string $Base64;

    private string $fileFrom;

    private  $allowTypes;

    private string $fileName;

    private string $Title;

    private ?string $extension = null;

    private ?string $fromDisk = null;

    private bool $preserved = false;

    private string $collection = "default";

    private array $customProperties = [];

    private string $disk = "public";

    private string $conversionDisk = "public";


    public function __construct($model)
    {
        $this->model = $model;
        $this->fileName = Str::random(10);
        return $this;
    }

    /**
     * set media source
     *
     * @param string $filefrom
     * @param string|null $fromDisk
     * @return self
     */
    public function setFrom(string $filefrom, string $fromDisk = null): self
    {
        $this->fileFrom =  $filefrom;
        $this->fromDisk = $fromDisk;
        return $this;
    }


    /**
     * set File key from request key
     *
     * @param string $key
     * @return self
     */
    public function setFile(string $key): self
    {
        $this->File = $key;
        return $this;
    }

    /**
     * set media Url
     *
     * @param string $Url
     * @param [type] ...$allowTypes
     * @return self
     */
    public function setUrl(string $Url, ...$allowTypes): self
    {
        $this->Url = $Url;
        $this->allowTypes = $allowTypes;
        return $this;
    }

    /**
     * set base 64 image
     *
     * @param string $ImageBase64
     * @return self
     */
    public function setBase64(string $ImageBase64): self
    {
        $this->Base64 = $ImageBase64;
        return $this;
    }


    /**
     * set Title of media
     *
     * @param string $Title
     * @return self
     */
    public function setTitle(string $Title): self
    {
        $this->Title = $Title;
        return $this;
    }

    /**
     * set media FileName
     *
     * @param string $fileName
     * @return self
     */
    public function setFileName(string $fileName): self
    {
        $this->fileName = $fileName;
        return $this;
    }

    /**
     * set media extendion
     *
     * @param string $extension
     * @return self
     */
    public function setExtension(string $extension): self
    {
        $this->extension = $extension;
        return $this;
    }
    /**
     * set media collection
     *
     * @param string $collection
     * @return self
     */
    public function setCollection(string $collection): self
    {
        $this->collection = $collection;
        return $this;
    }

    /**
     * set custom properties
     *
     * @param array $customProperties
     * @return self
     */
    public function setCustomProperties(array $customProperties): self
    {
        $this->customProperties = $customProperties;
        return $this;
    }

    /**
     * set media disk
     *
     * @param string $disk
     * @return self
     */
    public function setDisk(string $disk): self
    {
        $this->disk = $disk;
        return $this;
    }

    /**
     * Removing orginal file
     *
     * @param boolean $preserved
     * @return self
     */
    public function setPreserved(bool $preserved): self
    {
        $this->preserved = $preserved;
        return $this;
    }

    /**
     * Save media
     *@throws App\Exceptions\ImageServiceException
     * @return true
     */
    public function save(): bool
    {

        switch ($this->fileFrom) {
            case self::FILE_ADD_FROM_FILE:
                $File = $this->model->addMedia($this->File);
                break;
            case self::FILE_ADD_FROM_DISK:
                $File = $this->model->addMediafromDisk($this->File, $this->fromDisk);
                break;
            case self::FILE_FROM_INPUT:
                $File = $this->model->addMediaFromRequest($this->File);
                break;
            case self::FILE_FROM_BASE64:
                $File =  $this->model->addMediaFromBase64($this->Base64);
                break;
            case self::FILE_FROM_URL:
                $File =  $this->model->addMediaFromUrl($this->Url, $this->allowTypes);
                break;
            default:
                throw ImageServiceException::InvalidSource();
                break;
        }

        if ($this->extension) {
            $File->usingFileName($this->fileName . '.' . $this->extension);
        }

        if ($this->customProperties) {
            $File->withCustomProperties($this->customProperties);
        }

        if ($this->preserved) {
            $File->preservingOriginal();
        }

        $File->usingName($this->Title)->sanitizingFileName(function ($fileName) {
            return $this->fileName . '.' . pathinfo($fileName, PATHINFO_EXTENSION);
        })
            ->storingConversionsOnDisk($this->conversionDisk)
            ->toMediaCollection($this->collection, $this->disk);

        return true;
    }
}
